// Code generated by ent, DO NOT EDIT.

package ent

import (
	"context"
	"errors"
	"fmt"
	"sync"
	"time"

	"entgo.io/ent"
	"entgo.io/ent/dialect/sql"
	"kycnot.me/internal/ent/announcement"
	"kycnot.me/internal/ent/attribute"
	"kycnot.me/internal/ent/change"
	"kycnot.me/internal/ent/predicate"
	"kycnot.me/internal/ent/schema"
	"kycnot.me/internal/ent/service"
)

const (
	// Operation types.
	OpCreate    = ent.OpCreate
	OpDelete    = ent.OpDelete
	OpDeleteOne = ent.OpDeleteOne
	OpUpdate    = ent.OpUpdate
	OpUpdateOne = ent.OpUpdateOne

	// Node types.
	TypeAnnouncement = "Announcement"
	TypeAttribute    = "Attribute"
	TypeChange       = "Change"
	TypeService      = "Service"
)

// AnnouncementMutation represents an operation that mutates the Announcement nodes in the graph.
type AnnouncementMutation struct {
	config
	op            Op
	typ           string
	id            *int
	body          *string
	title         *string
	link          *string
	warning       *bool
	active        *bool
	clearedFields map[string]struct{}
	done          bool
	oldValue      func(context.Context) (*Announcement, error)
	predicates    []predicate.Announcement
}

var _ ent.Mutation = (*AnnouncementMutation)(nil)

// announcementOption allows management of the mutation configuration using functional options.
type announcementOption func(*AnnouncementMutation)

// newAnnouncementMutation creates new mutation for the Announcement entity.
func newAnnouncementMutation(c config, op Op, opts ...announcementOption) *AnnouncementMutation {
	m := &AnnouncementMutation{
		config:        c,
		op:            op,
		typ:           TypeAnnouncement,
		clearedFields: make(map[string]struct{}),
	}
	for _, opt := range opts {
		opt(m)
	}
	return m
}

// withAnnouncementID sets the ID field of the mutation.
func withAnnouncementID(id int) announcementOption {
	return func(m *AnnouncementMutation) {
		var (
			err   error
			once  sync.Once
			value *Announcement
		)
		m.oldValue = func(ctx context.Context) (*Announcement, error) {
			once.Do(func() {
				if m.done {
					err = errors.New("querying old values post mutation is not allowed")
				} else {
					value, err = m.Client().Announcement.Get(ctx, id)
				}
			})
			return value, err
		}
		m.id = &id
	}
}

// withAnnouncement sets the old Announcement of the mutation.
func withAnnouncement(node *Announcement) announcementOption {
	return func(m *AnnouncementMutation) {
		m.oldValue = func(context.Context) (*Announcement, error) {
			return node, nil
		}
		m.id = &node.ID
	}
}

// Client returns a new `ent.Client` from the mutation. If the mutation was
// executed in a transaction (ent.Tx), a transactional client is returned.
func (m AnnouncementMutation) Client() *Client {
	client := &Client{config: m.config}
	client.init()
	return client
}

// Tx returns an `ent.Tx` for mutations that were executed in transactions;
// it returns an error otherwise.
func (m AnnouncementMutation) Tx() (*Tx, error) {
	if _, ok := m.driver.(*txDriver); !ok {
		return nil, errors.New("ent: mutation is not running in a transaction")
	}
	tx := &Tx{config: m.config}
	tx.init()
	return tx, nil
}

// ID returns the ID value in the mutation. Note that the ID is only available
// if it was provided to the builder or after it was returned from the database.
func (m *AnnouncementMutation) ID() (id int, exists bool) {
	if m.id == nil {
		return
	}
	return *m.id, true
}

// IDs queries the database and returns the entity ids that match the mutation's predicate.
// That means, if the mutation is applied within a transaction with an isolation level such
// as sql.LevelSerializable, the returned ids match the ids of the rows that will be updated
// or updated by the mutation.
func (m *AnnouncementMutation) IDs(ctx context.Context) ([]int, error) {
	switch {
	case m.op.Is(OpUpdateOne | OpDeleteOne):
		id, exists := m.ID()
		if exists {
			return []int{id}, nil
		}
		fallthrough
	case m.op.Is(OpUpdate | OpDelete):
		return m.Client().Announcement.Query().Where(m.predicates...).IDs(ctx)
	default:
		return nil, fmt.Errorf("IDs is not allowed on %s operations", m.op)
	}
}

// SetBody sets the "body" field.
func (m *AnnouncementMutation) SetBody(s string) {
	m.body = &s
}

// Body returns the value of the "body" field in the mutation.
func (m *AnnouncementMutation) Body() (r string, exists bool) {
	v := m.body
	if v == nil {
		return
	}
	return *v, true
}

// OldBody returns the old "body" field's value of the Announcement entity.
// If the Announcement object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *AnnouncementMutation) OldBody(ctx context.Context) (v string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldBody is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldBody requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldBody: %w", err)
	}
	return oldValue.Body, nil
}

// ClearBody clears the value of the "body" field.
func (m *AnnouncementMutation) ClearBody() {
	m.body = nil
	m.clearedFields[announcement.FieldBody] = struct{}{}
}

// BodyCleared returns if the "body" field was cleared in this mutation.
func (m *AnnouncementMutation) BodyCleared() bool {
	_, ok := m.clearedFields[announcement.FieldBody]
	return ok
}

// ResetBody resets all changes to the "body" field.
func (m *AnnouncementMutation) ResetBody() {
	m.body = nil
	delete(m.clearedFields, announcement.FieldBody)
}

// SetTitle sets the "title" field.
func (m *AnnouncementMutation) SetTitle(s string) {
	m.title = &s
}

// Title returns the value of the "title" field in the mutation.
func (m *AnnouncementMutation) Title() (r string, exists bool) {
	v := m.title
	if v == nil {
		return
	}
	return *v, true
}

// OldTitle returns the old "title" field's value of the Announcement entity.
// If the Announcement object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *AnnouncementMutation) OldTitle(ctx context.Context) (v string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldTitle is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldTitle requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldTitle: %w", err)
	}
	return oldValue.Title, nil
}

// ClearTitle clears the value of the "title" field.
func (m *AnnouncementMutation) ClearTitle() {
	m.title = nil
	m.clearedFields[announcement.FieldTitle] = struct{}{}
}

// TitleCleared returns if the "title" field was cleared in this mutation.
func (m *AnnouncementMutation) TitleCleared() bool {
	_, ok := m.clearedFields[announcement.FieldTitle]
	return ok
}

// ResetTitle resets all changes to the "title" field.
func (m *AnnouncementMutation) ResetTitle() {
	m.title = nil
	delete(m.clearedFields, announcement.FieldTitle)
}

// SetLink sets the "link" field.
func (m *AnnouncementMutation) SetLink(s string) {
	m.link = &s
}

// Link returns the value of the "link" field in the mutation.
func (m *AnnouncementMutation) Link() (r string, exists bool) {
	v := m.link
	if v == nil {
		return
	}
	return *v, true
}

// OldLink returns the old "link" field's value of the Announcement entity.
// If the Announcement object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *AnnouncementMutation) OldLink(ctx context.Context) (v string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldLink is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldLink requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldLink: %w", err)
	}
	return oldValue.Link, nil
}

// ClearLink clears the value of the "link" field.
func (m *AnnouncementMutation) ClearLink() {
	m.link = nil
	m.clearedFields[announcement.FieldLink] = struct{}{}
}

// LinkCleared returns if the "link" field was cleared in this mutation.
func (m *AnnouncementMutation) LinkCleared() bool {
	_, ok := m.clearedFields[announcement.FieldLink]
	return ok
}

// ResetLink resets all changes to the "link" field.
func (m *AnnouncementMutation) ResetLink() {
	m.link = nil
	delete(m.clearedFields, announcement.FieldLink)
}

// SetWarning sets the "warning" field.
func (m *AnnouncementMutation) SetWarning(b bool) {
	m.warning = &b
}

// Warning returns the value of the "warning" field in the mutation.
func (m *AnnouncementMutation) Warning() (r bool, exists bool) {
	v := m.warning
	if v == nil {
		return
	}
	return *v, true
}

// OldWarning returns the old "warning" field's value of the Announcement entity.
// If the Announcement object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *AnnouncementMutation) OldWarning(ctx context.Context) (v bool, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldWarning is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldWarning requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldWarning: %w", err)
	}
	return oldValue.Warning, nil
}

// ClearWarning clears the value of the "warning" field.
func (m *AnnouncementMutation) ClearWarning() {
	m.warning = nil
	m.clearedFields[announcement.FieldWarning] = struct{}{}
}

// WarningCleared returns if the "warning" field was cleared in this mutation.
func (m *AnnouncementMutation) WarningCleared() bool {
	_, ok := m.clearedFields[announcement.FieldWarning]
	return ok
}

// ResetWarning resets all changes to the "warning" field.
func (m *AnnouncementMutation) ResetWarning() {
	m.warning = nil
	delete(m.clearedFields, announcement.FieldWarning)
}

// SetActive sets the "active" field.
func (m *AnnouncementMutation) SetActive(b bool) {
	m.active = &b
}

// Active returns the value of the "active" field in the mutation.
func (m *AnnouncementMutation) Active() (r bool, exists bool) {
	v := m.active
	if v == nil {
		return
	}
	return *v, true
}

// OldActive returns the old "active" field's value of the Announcement entity.
// If the Announcement object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *AnnouncementMutation) OldActive(ctx context.Context) (v bool, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldActive is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldActive requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldActive: %w", err)
	}
	return oldValue.Active, nil
}

// ClearActive clears the value of the "active" field.
func (m *AnnouncementMutation) ClearActive() {
	m.active = nil
	m.clearedFields[announcement.FieldActive] = struct{}{}
}

// ActiveCleared returns if the "active" field was cleared in this mutation.
func (m *AnnouncementMutation) ActiveCleared() bool {
	_, ok := m.clearedFields[announcement.FieldActive]
	return ok
}

// ResetActive resets all changes to the "active" field.
func (m *AnnouncementMutation) ResetActive() {
	m.active = nil
	delete(m.clearedFields, announcement.FieldActive)
}

// Where appends a list predicates to the AnnouncementMutation builder.
func (m *AnnouncementMutation) Where(ps ...predicate.Announcement) {
	m.predicates = append(m.predicates, ps...)
}

// WhereP appends storage-level predicates to the AnnouncementMutation builder. Using this method,
// users can use type-assertion to append predicates that do not depend on any generated package.
func (m *AnnouncementMutation) WhereP(ps ...func(*sql.Selector)) {
	p := make([]predicate.Announcement, len(ps))
	for i := range ps {
		p[i] = ps[i]
	}
	m.Where(p...)
}

// Op returns the operation name.
func (m *AnnouncementMutation) Op() Op {
	return m.op
}

// SetOp allows setting the mutation operation.
func (m *AnnouncementMutation) SetOp(op Op) {
	m.op = op
}

// Type returns the node type of this mutation (Announcement).
func (m *AnnouncementMutation) Type() string {
	return m.typ
}

// Fields returns all fields that were changed during this mutation. Note that in
// order to get all numeric fields that were incremented/decremented, call
// AddedFields().
func (m *AnnouncementMutation) Fields() []string {
	fields := make([]string, 0, 5)
	if m.body != nil {
		fields = append(fields, announcement.FieldBody)
	}
	if m.title != nil {
		fields = append(fields, announcement.FieldTitle)
	}
	if m.link != nil {
		fields = append(fields, announcement.FieldLink)
	}
	if m.warning != nil {
		fields = append(fields, announcement.FieldWarning)
	}
	if m.active != nil {
		fields = append(fields, announcement.FieldActive)
	}
	return fields
}

// Field returns the value of a field with the given name. The second boolean
// return value indicates that this field was not set, or was not defined in the
// schema.
func (m *AnnouncementMutation) Field(name string) (ent.Value, bool) {
	switch name {
	case announcement.FieldBody:
		return m.Body()
	case announcement.FieldTitle:
		return m.Title()
	case announcement.FieldLink:
		return m.Link()
	case announcement.FieldWarning:
		return m.Warning()
	case announcement.FieldActive:
		return m.Active()
	}
	return nil, false
}

// OldField returns the old value of the field from the database. An error is
// returned if the mutation operation is not UpdateOne, or the query to the
// database failed.
func (m *AnnouncementMutation) OldField(ctx context.Context, name string) (ent.Value, error) {
	switch name {
	case announcement.FieldBody:
		return m.OldBody(ctx)
	case announcement.FieldTitle:
		return m.OldTitle(ctx)
	case announcement.FieldLink:
		return m.OldLink(ctx)
	case announcement.FieldWarning:
		return m.OldWarning(ctx)
	case announcement.FieldActive:
		return m.OldActive(ctx)
	}
	return nil, fmt.Errorf("unknown Announcement field %s", name)
}

// SetField sets the value of a field with the given name. It returns an error if
// the field is not defined in the schema, or if the type mismatched the field
// type.
func (m *AnnouncementMutation) SetField(name string, value ent.Value) error {
	switch name {
	case announcement.FieldBody:
		v, ok := value.(string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetBody(v)
		return nil
	case announcement.FieldTitle:
		v, ok := value.(string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetTitle(v)
		return nil
	case announcement.FieldLink:
		v, ok := value.(string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetLink(v)
		return nil
	case announcement.FieldWarning:
		v, ok := value.(bool)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetWarning(v)
		return nil
	case announcement.FieldActive:
		v, ok := value.(bool)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetActive(v)
		return nil
	}
	return fmt.Errorf("unknown Announcement field %s", name)
}

// AddedFields returns all numeric fields that were incremented/decremented during
// this mutation.
func (m *AnnouncementMutation) AddedFields() []string {
	return nil
}

// AddedField returns the numeric value that was incremented/decremented on a field
// with the given name. The second boolean return value indicates that this field
// was not set, or was not defined in the schema.
func (m *AnnouncementMutation) AddedField(name string) (ent.Value, bool) {
	return nil, false
}

// AddField adds the value to the field with the given name. It returns an error if
// the field is not defined in the schema, or if the type mismatched the field
// type.
func (m *AnnouncementMutation) AddField(name string, value ent.Value) error {
	switch name {
	}
	return fmt.Errorf("unknown Announcement numeric field %s", name)
}

// ClearedFields returns all nullable fields that were cleared during this
// mutation.
func (m *AnnouncementMutation) ClearedFields() []string {
	var fields []string
	if m.FieldCleared(announcement.FieldBody) {
		fields = append(fields, announcement.FieldBody)
	}
	if m.FieldCleared(announcement.FieldTitle) {
		fields = append(fields, announcement.FieldTitle)
	}
	if m.FieldCleared(announcement.FieldLink) {
		fields = append(fields, announcement.FieldLink)
	}
	if m.FieldCleared(announcement.FieldWarning) {
		fields = append(fields, announcement.FieldWarning)
	}
	if m.FieldCleared(announcement.FieldActive) {
		fields = append(fields, announcement.FieldActive)
	}
	return fields
}

// FieldCleared returns a boolean indicating if a field with the given name was
// cleared in this mutation.
func (m *AnnouncementMutation) FieldCleared(name string) bool {
	_, ok := m.clearedFields[name]
	return ok
}

// ClearField clears the value of the field with the given name. It returns an
// error if the field is not defined in the schema.
func (m *AnnouncementMutation) ClearField(name string) error {
	switch name {
	case announcement.FieldBody:
		m.ClearBody()
		return nil
	case announcement.FieldTitle:
		m.ClearTitle()
		return nil
	case announcement.FieldLink:
		m.ClearLink()
		return nil
	case announcement.FieldWarning:
		m.ClearWarning()
		return nil
	case announcement.FieldActive:
		m.ClearActive()
		return nil
	}
	return fmt.Errorf("unknown Announcement nullable field %s", name)
}

// ResetField resets all changes in the mutation for the field with the given name.
// It returns an error if the field is not defined in the schema.
func (m *AnnouncementMutation) ResetField(name string) error {
	switch name {
	case announcement.FieldBody:
		m.ResetBody()
		return nil
	case announcement.FieldTitle:
		m.ResetTitle()
		return nil
	case announcement.FieldLink:
		m.ResetLink()
		return nil
	case announcement.FieldWarning:
		m.ResetWarning()
		return nil
	case announcement.FieldActive:
		m.ResetActive()
		return nil
	}
	return fmt.Errorf("unknown Announcement field %s", name)
}

// AddedEdges returns all edge names that were set/added in this mutation.
func (m *AnnouncementMutation) AddedEdges() []string {
	edges := make([]string, 0, 0)
	return edges
}

// AddedIDs returns all IDs (to other nodes) that were added for the given edge
// name in this mutation.
func (m *AnnouncementMutation) AddedIDs(name string) []ent.Value {
	return nil
}

// RemovedEdges returns all edge names that were removed in this mutation.
func (m *AnnouncementMutation) RemovedEdges() []string {
	edges := make([]string, 0, 0)
	return edges
}

// RemovedIDs returns all IDs (to other nodes) that were removed for the edge with
// the given name in this mutation.
func (m *AnnouncementMutation) RemovedIDs(name string) []ent.Value {
	return nil
}

// ClearedEdges returns all edge names that were cleared in this mutation.
func (m *AnnouncementMutation) ClearedEdges() []string {
	edges := make([]string, 0, 0)
	return edges
}

// EdgeCleared returns a boolean which indicates if the edge with the given name
// was cleared in this mutation.
func (m *AnnouncementMutation) EdgeCleared(name string) bool {
	return false
}

// ClearEdge clears the value of the edge with the given name. It returns an error
// if that edge is not defined in the schema.
func (m *AnnouncementMutation) ClearEdge(name string) error {
	return fmt.Errorf("unknown Announcement unique edge %s", name)
}

// ResetEdge resets all changes to the edge with the given name in this mutation.
// It returns an error if the edge is not defined in the schema.
func (m *AnnouncementMutation) ResetEdge(name string) error {
	return fmt.Errorf("unknown Announcement edge %s", name)
}

// AttributeMutation represents an operation that mutates the Attribute nodes in the graph.
type AttributeMutation struct {
	config
	op              Op
	typ             string
	id              *string
	createdAt       *time.Time
	title           *string
	bonus           *float64
	addbonus        *float64
	slug            *string
	rating          *attribute.Rating
	description     *string
	clearedFields   map[string]struct{}
	services        map[string]struct{}
	removedservices map[string]struct{}
	clearedservices bool
	done            bool
	oldValue        func(context.Context) (*Attribute, error)
	predicates      []predicate.Attribute
}

var _ ent.Mutation = (*AttributeMutation)(nil)

// attributeOption allows management of the mutation configuration using functional options.
type attributeOption func(*AttributeMutation)

// newAttributeMutation creates new mutation for the Attribute entity.
func newAttributeMutation(c config, op Op, opts ...attributeOption) *AttributeMutation {
	m := &AttributeMutation{
		config:        c,
		op:            op,
		typ:           TypeAttribute,
		clearedFields: make(map[string]struct{}),
	}
	for _, opt := range opts {
		opt(m)
	}
	return m
}

// withAttributeID sets the ID field of the mutation.
func withAttributeID(id string) attributeOption {
	return func(m *AttributeMutation) {
		var (
			err   error
			once  sync.Once
			value *Attribute
		)
		m.oldValue = func(ctx context.Context) (*Attribute, error) {
			once.Do(func() {
				if m.done {
					err = errors.New("querying old values post mutation is not allowed")
				} else {
					value, err = m.Client().Attribute.Get(ctx, id)
				}
			})
			return value, err
		}
		m.id = &id
	}
}

// withAttribute sets the old Attribute of the mutation.
func withAttribute(node *Attribute) attributeOption {
	return func(m *AttributeMutation) {
		m.oldValue = func(context.Context) (*Attribute, error) {
			return node, nil
		}
		m.id = &node.ID
	}
}

// Client returns a new `ent.Client` from the mutation. If the mutation was
// executed in a transaction (ent.Tx), a transactional client is returned.
func (m AttributeMutation) Client() *Client {
	client := &Client{config: m.config}
	client.init()
	return client
}

// Tx returns an `ent.Tx` for mutations that were executed in transactions;
// it returns an error otherwise.
func (m AttributeMutation) Tx() (*Tx, error) {
	if _, ok := m.driver.(*txDriver); !ok {
		return nil, errors.New("ent: mutation is not running in a transaction")
	}
	tx := &Tx{config: m.config}
	tx.init()
	return tx, nil
}

// SetID sets the value of the id field. Note that this
// operation is only accepted on creation of Attribute entities.
func (m *AttributeMutation) SetID(id string) {
	m.id = &id
}

// ID returns the ID value in the mutation. Note that the ID is only available
// if it was provided to the builder or after it was returned from the database.
func (m *AttributeMutation) ID() (id string, exists bool) {
	if m.id == nil {
		return
	}
	return *m.id, true
}

// IDs queries the database and returns the entity ids that match the mutation's predicate.
// That means, if the mutation is applied within a transaction with an isolation level such
// as sql.LevelSerializable, the returned ids match the ids of the rows that will be updated
// or updated by the mutation.
func (m *AttributeMutation) IDs(ctx context.Context) ([]string, error) {
	switch {
	case m.op.Is(OpUpdateOne | OpDeleteOne):
		id, exists := m.ID()
		if exists {
			return []string{id}, nil
		}
		fallthrough
	case m.op.Is(OpUpdate | OpDelete):
		return m.Client().Attribute.Query().Where(m.predicates...).IDs(ctx)
	default:
		return nil, fmt.Errorf("IDs is not allowed on %s operations", m.op)
	}
}

// SetCreatedAt sets the "createdAt" field.
func (m *AttributeMutation) SetCreatedAt(t time.Time) {
	m.createdAt = &t
}

// CreatedAt returns the value of the "createdAt" field in the mutation.
func (m *AttributeMutation) CreatedAt() (r time.Time, exists bool) {
	v := m.createdAt
	if v == nil {
		return
	}
	return *v, true
}

// OldCreatedAt returns the old "createdAt" field's value of the Attribute entity.
// If the Attribute object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *AttributeMutation) OldCreatedAt(ctx context.Context) (v time.Time, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldCreatedAt is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldCreatedAt requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldCreatedAt: %w", err)
	}
	return oldValue.CreatedAt, nil
}

// ResetCreatedAt resets all changes to the "createdAt" field.
func (m *AttributeMutation) ResetCreatedAt() {
	m.createdAt = nil
}

// SetTitle sets the "title" field.
func (m *AttributeMutation) SetTitle(s string) {
	m.title = &s
}

// Title returns the value of the "title" field in the mutation.
func (m *AttributeMutation) Title() (r string, exists bool) {
	v := m.title
	if v == nil {
		return
	}
	return *v, true
}

// OldTitle returns the old "title" field's value of the Attribute entity.
// If the Attribute object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *AttributeMutation) OldTitle(ctx context.Context) (v string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldTitle is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldTitle requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldTitle: %w", err)
	}
	return oldValue.Title, nil
}

// ResetTitle resets all changes to the "title" field.
func (m *AttributeMutation) ResetTitle() {
	m.title = nil
}

// SetBonus sets the "bonus" field.
func (m *AttributeMutation) SetBonus(f float64) {
	m.bonus = &f
	m.addbonus = nil
}

// Bonus returns the value of the "bonus" field in the mutation.
func (m *AttributeMutation) Bonus() (r float64, exists bool) {
	v := m.bonus
	if v == nil {
		return
	}
	return *v, true
}

// OldBonus returns the old "bonus" field's value of the Attribute entity.
// If the Attribute object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *AttributeMutation) OldBonus(ctx context.Context) (v float64, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldBonus is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldBonus requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldBonus: %w", err)
	}
	return oldValue.Bonus, nil
}

// AddBonus adds f to the "bonus" field.
func (m *AttributeMutation) AddBonus(f float64) {
	if m.addbonus != nil {
		*m.addbonus += f
	} else {
		m.addbonus = &f
	}
}

// AddedBonus returns the value that was added to the "bonus" field in this mutation.
func (m *AttributeMutation) AddedBonus() (r float64, exists bool) {
	v := m.addbonus
	if v == nil {
		return
	}
	return *v, true
}

// ResetBonus resets all changes to the "bonus" field.
func (m *AttributeMutation) ResetBonus() {
	m.bonus = nil
	m.addbonus = nil
}

// SetSlug sets the "slug" field.
func (m *AttributeMutation) SetSlug(s string) {
	m.slug = &s
}

// Slug returns the value of the "slug" field in the mutation.
func (m *AttributeMutation) Slug() (r string, exists bool) {
	v := m.slug
	if v == nil {
		return
	}
	return *v, true
}

// OldSlug returns the old "slug" field's value of the Attribute entity.
// If the Attribute object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *AttributeMutation) OldSlug(ctx context.Context) (v string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldSlug is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldSlug requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldSlug: %w", err)
	}
	return oldValue.Slug, nil
}

// ResetSlug resets all changes to the "slug" field.
func (m *AttributeMutation) ResetSlug() {
	m.slug = nil
}

// SetRating sets the "rating" field.
func (m *AttributeMutation) SetRating(a attribute.Rating) {
	m.rating = &a
}

// Rating returns the value of the "rating" field in the mutation.
func (m *AttributeMutation) Rating() (r attribute.Rating, exists bool) {
	v := m.rating
	if v == nil {
		return
	}
	return *v, true
}

// OldRating returns the old "rating" field's value of the Attribute entity.
// If the Attribute object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *AttributeMutation) OldRating(ctx context.Context) (v attribute.Rating, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldRating is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldRating requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldRating: %w", err)
	}
	return oldValue.Rating, nil
}

// ResetRating resets all changes to the "rating" field.
func (m *AttributeMutation) ResetRating() {
	m.rating = nil
}

// SetDescription sets the "description" field.
func (m *AttributeMutation) SetDescription(s string) {
	m.description = &s
}

// Description returns the value of the "description" field in the mutation.
func (m *AttributeMutation) Description() (r string, exists bool) {
	v := m.description
	if v == nil {
		return
	}
	return *v, true
}

// OldDescription returns the old "description" field's value of the Attribute entity.
// If the Attribute object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *AttributeMutation) OldDescription(ctx context.Context) (v string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldDescription is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldDescription requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldDescription: %w", err)
	}
	return oldValue.Description, nil
}

// ClearDescription clears the value of the "description" field.
func (m *AttributeMutation) ClearDescription() {
	m.description = nil
	m.clearedFields[attribute.FieldDescription] = struct{}{}
}

// DescriptionCleared returns if the "description" field was cleared in this mutation.
func (m *AttributeMutation) DescriptionCleared() bool {
	_, ok := m.clearedFields[attribute.FieldDescription]
	return ok
}

// ResetDescription resets all changes to the "description" field.
func (m *AttributeMutation) ResetDescription() {
	m.description = nil
	delete(m.clearedFields, attribute.FieldDescription)
}

// AddServiceIDs adds the "services" edge to the Service entity by ids.
func (m *AttributeMutation) AddServiceIDs(ids ...string) {
	if m.services == nil {
		m.services = make(map[string]struct{})
	}
	for i := range ids {
		m.services[ids[i]] = struct{}{}
	}
}

// ClearServices clears the "services" edge to the Service entity.
func (m *AttributeMutation) ClearServices() {
	m.clearedservices = true
}

// ServicesCleared reports if the "services" edge to the Service entity was cleared.
func (m *AttributeMutation) ServicesCleared() bool {
	return m.clearedservices
}

// RemoveServiceIDs removes the "services" edge to the Service entity by IDs.
func (m *AttributeMutation) RemoveServiceIDs(ids ...string) {
	if m.removedservices == nil {
		m.removedservices = make(map[string]struct{})
	}
	for i := range ids {
		delete(m.services, ids[i])
		m.removedservices[ids[i]] = struct{}{}
	}
}

// RemovedServices returns the removed IDs of the "services" edge to the Service entity.
func (m *AttributeMutation) RemovedServicesIDs() (ids []string) {
	for id := range m.removedservices {
		ids = append(ids, id)
	}
	return
}

// ServicesIDs returns the "services" edge IDs in the mutation.
func (m *AttributeMutation) ServicesIDs() (ids []string) {
	for id := range m.services {
		ids = append(ids, id)
	}
	return
}

// ResetServices resets all changes to the "services" edge.
func (m *AttributeMutation) ResetServices() {
	m.services = nil
	m.clearedservices = false
	m.removedservices = nil
}

// Where appends a list predicates to the AttributeMutation builder.
func (m *AttributeMutation) Where(ps ...predicate.Attribute) {
	m.predicates = append(m.predicates, ps...)
}

// WhereP appends storage-level predicates to the AttributeMutation builder. Using this method,
// users can use type-assertion to append predicates that do not depend on any generated package.
func (m *AttributeMutation) WhereP(ps ...func(*sql.Selector)) {
	p := make([]predicate.Attribute, len(ps))
	for i := range ps {
		p[i] = ps[i]
	}
	m.Where(p...)
}

// Op returns the operation name.
func (m *AttributeMutation) Op() Op {
	return m.op
}

// SetOp allows setting the mutation operation.
func (m *AttributeMutation) SetOp(op Op) {
	m.op = op
}

// Type returns the node type of this mutation (Attribute).
func (m *AttributeMutation) Type() string {
	return m.typ
}

// Fields returns all fields that were changed during this mutation. Note that in
// order to get all numeric fields that were incremented/decremented, call
// AddedFields().
func (m *AttributeMutation) Fields() []string {
	fields := make([]string, 0, 6)
	if m.createdAt != nil {
		fields = append(fields, attribute.FieldCreatedAt)
	}
	if m.title != nil {
		fields = append(fields, attribute.FieldTitle)
	}
	if m.bonus != nil {
		fields = append(fields, attribute.FieldBonus)
	}
	if m.slug != nil {
		fields = append(fields, attribute.FieldSlug)
	}
	if m.rating != nil {
		fields = append(fields, attribute.FieldRating)
	}
	if m.description != nil {
		fields = append(fields, attribute.FieldDescription)
	}
	return fields
}

// Field returns the value of a field with the given name. The second boolean
// return value indicates that this field was not set, or was not defined in the
// schema.
func (m *AttributeMutation) Field(name string) (ent.Value, bool) {
	switch name {
	case attribute.FieldCreatedAt:
		return m.CreatedAt()
	case attribute.FieldTitle:
		return m.Title()
	case attribute.FieldBonus:
		return m.Bonus()
	case attribute.FieldSlug:
		return m.Slug()
	case attribute.FieldRating:
		return m.Rating()
	case attribute.FieldDescription:
		return m.Description()
	}
	return nil, false
}

// OldField returns the old value of the field from the database. An error is
// returned if the mutation operation is not UpdateOne, or the query to the
// database failed.
func (m *AttributeMutation) OldField(ctx context.Context, name string) (ent.Value, error) {
	switch name {
	case attribute.FieldCreatedAt:
		return m.OldCreatedAt(ctx)
	case attribute.FieldTitle:
		return m.OldTitle(ctx)
	case attribute.FieldBonus:
		return m.OldBonus(ctx)
	case attribute.FieldSlug:
		return m.OldSlug(ctx)
	case attribute.FieldRating:
		return m.OldRating(ctx)
	case attribute.FieldDescription:
		return m.OldDescription(ctx)
	}
	return nil, fmt.Errorf("unknown Attribute field %s", name)
}

// SetField sets the value of a field with the given name. It returns an error if
// the field is not defined in the schema, or if the type mismatched the field
// type.
func (m *AttributeMutation) SetField(name string, value ent.Value) error {
	switch name {
	case attribute.FieldCreatedAt:
		v, ok := value.(time.Time)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetCreatedAt(v)
		return nil
	case attribute.FieldTitle:
		v, ok := value.(string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetTitle(v)
		return nil
	case attribute.FieldBonus:
		v, ok := value.(float64)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetBonus(v)
		return nil
	case attribute.FieldSlug:
		v, ok := value.(string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetSlug(v)
		return nil
	case attribute.FieldRating:
		v, ok := value.(attribute.Rating)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetRating(v)
		return nil
	case attribute.FieldDescription:
		v, ok := value.(string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetDescription(v)
		return nil
	}
	return fmt.Errorf("unknown Attribute field %s", name)
}

// AddedFields returns all numeric fields that were incremented/decremented during
// this mutation.
func (m *AttributeMutation) AddedFields() []string {
	var fields []string
	if m.addbonus != nil {
		fields = append(fields, attribute.FieldBonus)
	}
	return fields
}

// AddedField returns the numeric value that was incremented/decremented on a field
// with the given name. The second boolean return value indicates that this field
// was not set, or was not defined in the schema.
func (m *AttributeMutation) AddedField(name string) (ent.Value, bool) {
	switch name {
	case attribute.FieldBonus:
		return m.AddedBonus()
	}
	return nil, false
}

// AddField adds the value to the field with the given name. It returns an error if
// the field is not defined in the schema, or if the type mismatched the field
// type.
func (m *AttributeMutation) AddField(name string, value ent.Value) error {
	switch name {
	case attribute.FieldBonus:
		v, ok := value.(float64)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.AddBonus(v)
		return nil
	}
	return fmt.Errorf("unknown Attribute numeric field %s", name)
}

// ClearedFields returns all nullable fields that were cleared during this
// mutation.
func (m *AttributeMutation) ClearedFields() []string {
	var fields []string
	if m.FieldCleared(attribute.FieldDescription) {
		fields = append(fields, attribute.FieldDescription)
	}
	return fields
}

// FieldCleared returns a boolean indicating if a field with the given name was
// cleared in this mutation.
func (m *AttributeMutation) FieldCleared(name string) bool {
	_, ok := m.clearedFields[name]
	return ok
}

// ClearField clears the value of the field with the given name. It returns an
// error if the field is not defined in the schema.
func (m *AttributeMutation) ClearField(name string) error {
	switch name {
	case attribute.FieldDescription:
		m.ClearDescription()
		return nil
	}
	return fmt.Errorf("unknown Attribute nullable field %s", name)
}

// ResetField resets all changes in the mutation for the field with the given name.
// It returns an error if the field is not defined in the schema.
func (m *AttributeMutation) ResetField(name string) error {
	switch name {
	case attribute.FieldCreatedAt:
		m.ResetCreatedAt()
		return nil
	case attribute.FieldTitle:
		m.ResetTitle()
		return nil
	case attribute.FieldBonus:
		m.ResetBonus()
		return nil
	case attribute.FieldSlug:
		m.ResetSlug()
		return nil
	case attribute.FieldRating:
		m.ResetRating()
		return nil
	case attribute.FieldDescription:
		m.ResetDescription()
		return nil
	}
	return fmt.Errorf("unknown Attribute field %s", name)
}

// AddedEdges returns all edge names that were set/added in this mutation.
func (m *AttributeMutation) AddedEdges() []string {
	edges := make([]string, 0, 1)
	if m.services != nil {
		edges = append(edges, attribute.EdgeServices)
	}
	return edges
}

// AddedIDs returns all IDs (to other nodes) that were added for the given edge
// name in this mutation.
func (m *AttributeMutation) AddedIDs(name string) []ent.Value {
	switch name {
	case attribute.EdgeServices:
		ids := make([]ent.Value, 0, len(m.services))
		for id := range m.services {
			ids = append(ids, id)
		}
		return ids
	}
	return nil
}

// RemovedEdges returns all edge names that were removed in this mutation.
func (m *AttributeMutation) RemovedEdges() []string {
	edges := make([]string, 0, 1)
	if m.removedservices != nil {
		edges = append(edges, attribute.EdgeServices)
	}
	return edges
}

// RemovedIDs returns all IDs (to other nodes) that were removed for the edge with
// the given name in this mutation.
func (m *AttributeMutation) RemovedIDs(name string) []ent.Value {
	switch name {
	case attribute.EdgeServices:
		ids := make([]ent.Value, 0, len(m.removedservices))
		for id := range m.removedservices {
			ids = append(ids, id)
		}
		return ids
	}
	return nil
}

// ClearedEdges returns all edge names that were cleared in this mutation.
func (m *AttributeMutation) ClearedEdges() []string {
	edges := make([]string, 0, 1)
	if m.clearedservices {
		edges = append(edges, attribute.EdgeServices)
	}
	return edges
}

// EdgeCleared returns a boolean which indicates if the edge with the given name
// was cleared in this mutation.
func (m *AttributeMutation) EdgeCleared(name string) bool {
	switch name {
	case attribute.EdgeServices:
		return m.clearedservices
	}
	return false
}

// ClearEdge clears the value of the edge with the given name. It returns an error
// if that edge is not defined in the schema.
func (m *AttributeMutation) ClearEdge(name string) error {
	switch name {
	}
	return fmt.Errorf("unknown Attribute unique edge %s", name)
}

// ResetEdge resets all changes to the edge with the given name in this mutation.
// It returns an error if the edge is not defined in the schema.
func (m *AttributeMutation) ResetEdge(name string) error {
	switch name {
	case attribute.EdgeServices:
		m.ResetServices()
		return nil
	}
	return fmt.Errorf("unknown Attribute edge %s", name)
}

// ChangeMutation represents an operation that mutates the Change nodes in the graph.
type ChangeMutation struct {
	config
	op             Op
	typ            string
	id             *int
	_Title         *string
	_Summary       *string
	createdAt      *time.Time
	warning        *bool
	clearedFields  map[string]struct{}
	service        *string
	clearedservice bool
	done           bool
	oldValue       func(context.Context) (*Change, error)
	predicates     []predicate.Change
}

var _ ent.Mutation = (*ChangeMutation)(nil)

// changeOption allows management of the mutation configuration using functional options.
type changeOption func(*ChangeMutation)

// newChangeMutation creates new mutation for the Change entity.
func newChangeMutation(c config, op Op, opts ...changeOption) *ChangeMutation {
	m := &ChangeMutation{
		config:        c,
		op:            op,
		typ:           TypeChange,
		clearedFields: make(map[string]struct{}),
	}
	for _, opt := range opts {
		opt(m)
	}
	return m
}

// withChangeID sets the ID field of the mutation.
func withChangeID(id int) changeOption {
	return func(m *ChangeMutation) {
		var (
			err   error
			once  sync.Once
			value *Change
		)
		m.oldValue = func(ctx context.Context) (*Change, error) {
			once.Do(func() {
				if m.done {
					err = errors.New("querying old values post mutation is not allowed")
				} else {
					value, err = m.Client().Change.Get(ctx, id)
				}
			})
			return value, err
		}
		m.id = &id
	}
}

// withChange sets the old Change of the mutation.
func withChange(node *Change) changeOption {
	return func(m *ChangeMutation) {
		m.oldValue = func(context.Context) (*Change, error) {
			return node, nil
		}
		m.id = &node.ID
	}
}

// Client returns a new `ent.Client` from the mutation. If the mutation was
// executed in a transaction (ent.Tx), a transactional client is returned.
func (m ChangeMutation) Client() *Client {
	client := &Client{config: m.config}
	client.init()
	return client
}

// Tx returns an `ent.Tx` for mutations that were executed in transactions;
// it returns an error otherwise.
func (m ChangeMutation) Tx() (*Tx, error) {
	if _, ok := m.driver.(*txDriver); !ok {
		return nil, errors.New("ent: mutation is not running in a transaction")
	}
	tx := &Tx{config: m.config}
	tx.init()
	return tx, nil
}

// ID returns the ID value in the mutation. Note that the ID is only available
// if it was provided to the builder or after it was returned from the database.
func (m *ChangeMutation) ID() (id int, exists bool) {
	if m.id == nil {
		return
	}
	return *m.id, true
}

// IDs queries the database and returns the entity ids that match the mutation's predicate.
// That means, if the mutation is applied within a transaction with an isolation level such
// as sql.LevelSerializable, the returned ids match the ids of the rows that will be updated
// or updated by the mutation.
func (m *ChangeMutation) IDs(ctx context.Context) ([]int, error) {
	switch {
	case m.op.Is(OpUpdateOne | OpDeleteOne):
		id, exists := m.ID()
		if exists {
			return []int{id}, nil
		}
		fallthrough
	case m.op.Is(OpUpdate | OpDelete):
		return m.Client().Change.Query().Where(m.predicates...).IDs(ctx)
	default:
		return nil, fmt.Errorf("IDs is not allowed on %s operations", m.op)
	}
}

// SetTitle sets the "Title" field.
func (m *ChangeMutation) SetTitle(s string) {
	m._Title = &s
}

// Title returns the value of the "Title" field in the mutation.
func (m *ChangeMutation) Title() (r string, exists bool) {
	v := m._Title
	if v == nil {
		return
	}
	return *v, true
}

// OldTitle returns the old "Title" field's value of the Change entity.
// If the Change object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ChangeMutation) OldTitle(ctx context.Context) (v string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldTitle is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldTitle requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldTitle: %w", err)
	}
	return oldValue.Title, nil
}

// ClearTitle clears the value of the "Title" field.
func (m *ChangeMutation) ClearTitle() {
	m._Title = nil
	m.clearedFields[change.FieldTitle] = struct{}{}
}

// TitleCleared returns if the "Title" field was cleared in this mutation.
func (m *ChangeMutation) TitleCleared() bool {
	_, ok := m.clearedFields[change.FieldTitle]
	return ok
}

// ResetTitle resets all changes to the "Title" field.
func (m *ChangeMutation) ResetTitle() {
	m._Title = nil
	delete(m.clearedFields, change.FieldTitle)
}

// SetSummary sets the "Summary" field.
func (m *ChangeMutation) SetSummary(s string) {
	m._Summary = &s
}

// Summary returns the value of the "Summary" field in the mutation.
func (m *ChangeMutation) Summary() (r string, exists bool) {
	v := m._Summary
	if v == nil {
		return
	}
	return *v, true
}

// OldSummary returns the old "Summary" field's value of the Change entity.
// If the Change object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ChangeMutation) OldSummary(ctx context.Context) (v string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldSummary is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldSummary requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldSummary: %w", err)
	}
	return oldValue.Summary, nil
}

// ClearSummary clears the value of the "Summary" field.
func (m *ChangeMutation) ClearSummary() {
	m._Summary = nil
	m.clearedFields[change.FieldSummary] = struct{}{}
}

// SummaryCleared returns if the "Summary" field was cleared in this mutation.
func (m *ChangeMutation) SummaryCleared() bool {
	_, ok := m.clearedFields[change.FieldSummary]
	return ok
}

// ResetSummary resets all changes to the "Summary" field.
func (m *ChangeMutation) ResetSummary() {
	m._Summary = nil
	delete(m.clearedFields, change.FieldSummary)
}

// SetCreatedAt sets the "createdAt" field.
func (m *ChangeMutation) SetCreatedAt(t time.Time) {
	m.createdAt = &t
}

// CreatedAt returns the value of the "createdAt" field in the mutation.
func (m *ChangeMutation) CreatedAt() (r time.Time, exists bool) {
	v := m.createdAt
	if v == nil {
		return
	}
	return *v, true
}

// OldCreatedAt returns the old "createdAt" field's value of the Change entity.
// If the Change object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ChangeMutation) OldCreatedAt(ctx context.Context) (v time.Time, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldCreatedAt is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldCreatedAt requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldCreatedAt: %w", err)
	}
	return oldValue.CreatedAt, nil
}

// ResetCreatedAt resets all changes to the "createdAt" field.
func (m *ChangeMutation) ResetCreatedAt() {
	m.createdAt = nil
}

// SetWarning sets the "warning" field.
func (m *ChangeMutation) SetWarning(b bool) {
	m.warning = &b
}

// Warning returns the value of the "warning" field in the mutation.
func (m *ChangeMutation) Warning() (r bool, exists bool) {
	v := m.warning
	if v == nil {
		return
	}
	return *v, true
}

// OldWarning returns the old "warning" field's value of the Change entity.
// If the Change object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ChangeMutation) OldWarning(ctx context.Context) (v bool, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldWarning is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldWarning requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldWarning: %w", err)
	}
	return oldValue.Warning, nil
}

// ResetWarning resets all changes to the "warning" field.
func (m *ChangeMutation) ResetWarning() {
	m.warning = nil
}

// SetServiceID sets the "service" edge to the Service entity by id.
func (m *ChangeMutation) SetServiceID(id string) {
	m.service = &id
}

// ClearService clears the "service" edge to the Service entity.
func (m *ChangeMutation) ClearService() {
	m.clearedservice = true
}

// ServiceCleared reports if the "service" edge to the Service entity was cleared.
func (m *ChangeMutation) ServiceCleared() bool {
	return m.clearedservice
}

// ServiceID returns the "service" edge ID in the mutation.
func (m *ChangeMutation) ServiceID() (id string, exists bool) {
	if m.service != nil {
		return *m.service, true
	}
	return
}

// ServiceIDs returns the "service" edge IDs in the mutation.
// Note that IDs always returns len(IDs) <= 1 for unique edges, and you should use
// ServiceID instead. It exists only for internal usage by the builders.
func (m *ChangeMutation) ServiceIDs() (ids []string) {
	if id := m.service; id != nil {
		ids = append(ids, *id)
	}
	return
}

// ResetService resets all changes to the "service" edge.
func (m *ChangeMutation) ResetService() {
	m.service = nil
	m.clearedservice = false
}

// Where appends a list predicates to the ChangeMutation builder.
func (m *ChangeMutation) Where(ps ...predicate.Change) {
	m.predicates = append(m.predicates, ps...)
}

// WhereP appends storage-level predicates to the ChangeMutation builder. Using this method,
// users can use type-assertion to append predicates that do not depend on any generated package.
func (m *ChangeMutation) WhereP(ps ...func(*sql.Selector)) {
	p := make([]predicate.Change, len(ps))
	for i := range ps {
		p[i] = ps[i]
	}
	m.Where(p...)
}

// Op returns the operation name.
func (m *ChangeMutation) Op() Op {
	return m.op
}

// SetOp allows setting the mutation operation.
func (m *ChangeMutation) SetOp(op Op) {
	m.op = op
}

// Type returns the node type of this mutation (Change).
func (m *ChangeMutation) Type() string {
	return m.typ
}

// Fields returns all fields that were changed during this mutation. Note that in
// order to get all numeric fields that were incremented/decremented, call
// AddedFields().
func (m *ChangeMutation) Fields() []string {
	fields := make([]string, 0, 4)
	if m._Title != nil {
		fields = append(fields, change.FieldTitle)
	}
	if m._Summary != nil {
		fields = append(fields, change.FieldSummary)
	}
	if m.createdAt != nil {
		fields = append(fields, change.FieldCreatedAt)
	}
	if m.warning != nil {
		fields = append(fields, change.FieldWarning)
	}
	return fields
}

// Field returns the value of a field with the given name. The second boolean
// return value indicates that this field was not set, or was not defined in the
// schema.
func (m *ChangeMutation) Field(name string) (ent.Value, bool) {
	switch name {
	case change.FieldTitle:
		return m.Title()
	case change.FieldSummary:
		return m.Summary()
	case change.FieldCreatedAt:
		return m.CreatedAt()
	case change.FieldWarning:
		return m.Warning()
	}
	return nil, false
}

// OldField returns the old value of the field from the database. An error is
// returned if the mutation operation is not UpdateOne, or the query to the
// database failed.
func (m *ChangeMutation) OldField(ctx context.Context, name string) (ent.Value, error) {
	switch name {
	case change.FieldTitle:
		return m.OldTitle(ctx)
	case change.FieldSummary:
		return m.OldSummary(ctx)
	case change.FieldCreatedAt:
		return m.OldCreatedAt(ctx)
	case change.FieldWarning:
		return m.OldWarning(ctx)
	}
	return nil, fmt.Errorf("unknown Change field %s", name)
}

// SetField sets the value of a field with the given name. It returns an error if
// the field is not defined in the schema, or if the type mismatched the field
// type.
func (m *ChangeMutation) SetField(name string, value ent.Value) error {
	switch name {
	case change.FieldTitle:
		v, ok := value.(string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetTitle(v)
		return nil
	case change.FieldSummary:
		v, ok := value.(string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetSummary(v)
		return nil
	case change.FieldCreatedAt:
		v, ok := value.(time.Time)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetCreatedAt(v)
		return nil
	case change.FieldWarning:
		v, ok := value.(bool)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetWarning(v)
		return nil
	}
	return fmt.Errorf("unknown Change field %s", name)
}

// AddedFields returns all numeric fields that were incremented/decremented during
// this mutation.
func (m *ChangeMutation) AddedFields() []string {
	return nil
}

// AddedField returns the numeric value that was incremented/decremented on a field
// with the given name. The second boolean return value indicates that this field
// was not set, or was not defined in the schema.
func (m *ChangeMutation) AddedField(name string) (ent.Value, bool) {
	return nil, false
}

// AddField adds the value to the field with the given name. It returns an error if
// the field is not defined in the schema, or if the type mismatched the field
// type.
func (m *ChangeMutation) AddField(name string, value ent.Value) error {
	switch name {
	}
	return fmt.Errorf("unknown Change numeric field %s", name)
}

// ClearedFields returns all nullable fields that were cleared during this
// mutation.
func (m *ChangeMutation) ClearedFields() []string {
	var fields []string
	if m.FieldCleared(change.FieldTitle) {
		fields = append(fields, change.FieldTitle)
	}
	if m.FieldCleared(change.FieldSummary) {
		fields = append(fields, change.FieldSummary)
	}
	return fields
}

// FieldCleared returns a boolean indicating if a field with the given name was
// cleared in this mutation.
func (m *ChangeMutation) FieldCleared(name string) bool {
	_, ok := m.clearedFields[name]
	return ok
}

// ClearField clears the value of the field with the given name. It returns an
// error if the field is not defined in the schema.
func (m *ChangeMutation) ClearField(name string) error {
	switch name {
	case change.FieldTitle:
		m.ClearTitle()
		return nil
	case change.FieldSummary:
		m.ClearSummary()
		return nil
	}
	return fmt.Errorf("unknown Change nullable field %s", name)
}

// ResetField resets all changes in the mutation for the field with the given name.
// It returns an error if the field is not defined in the schema.
func (m *ChangeMutation) ResetField(name string) error {
	switch name {
	case change.FieldTitle:
		m.ResetTitle()
		return nil
	case change.FieldSummary:
		m.ResetSummary()
		return nil
	case change.FieldCreatedAt:
		m.ResetCreatedAt()
		return nil
	case change.FieldWarning:
		m.ResetWarning()
		return nil
	}
	return fmt.Errorf("unknown Change field %s", name)
}

// AddedEdges returns all edge names that were set/added in this mutation.
func (m *ChangeMutation) AddedEdges() []string {
	edges := make([]string, 0, 1)
	if m.service != nil {
		edges = append(edges, change.EdgeService)
	}
	return edges
}

// AddedIDs returns all IDs (to other nodes) that were added for the given edge
// name in this mutation.
func (m *ChangeMutation) AddedIDs(name string) []ent.Value {
	switch name {
	case change.EdgeService:
		if id := m.service; id != nil {
			return []ent.Value{*id}
		}
	}
	return nil
}

// RemovedEdges returns all edge names that were removed in this mutation.
func (m *ChangeMutation) RemovedEdges() []string {
	edges := make([]string, 0, 1)
	return edges
}

// RemovedIDs returns all IDs (to other nodes) that were removed for the edge with
// the given name in this mutation.
func (m *ChangeMutation) RemovedIDs(name string) []ent.Value {
	return nil
}

// ClearedEdges returns all edge names that were cleared in this mutation.
func (m *ChangeMutation) ClearedEdges() []string {
	edges := make([]string, 0, 1)
	if m.clearedservice {
		edges = append(edges, change.EdgeService)
	}
	return edges
}

// EdgeCleared returns a boolean which indicates if the edge with the given name
// was cleared in this mutation.
func (m *ChangeMutation) EdgeCleared(name string) bool {
	switch name {
	case change.EdgeService:
		return m.clearedservice
	}
	return false
}

// ClearEdge clears the value of the edge with the given name. It returns an error
// if that edge is not defined in the schema.
func (m *ChangeMutation) ClearEdge(name string) error {
	switch name {
	case change.EdgeService:
		m.ClearService()
		return nil
	}
	return fmt.Errorf("unknown Change unique edge %s", name)
}

// ResetEdge resets all changes to the edge with the given name in this mutation.
// It returns an error if the edge is not defined in the schema.
func (m *ChangeMutation) ResetEdge(name string) error {
	switch name {
	case change.EdgeService:
		m.ResetService()
		return nil
	}
	return fmt.Errorf("unknown Change edge %s", name)
}

// ServiceMutation represents an operation that mutates the Service nodes in the graph.
type ServiceMutation struct {
	config
	op                Op
	typ               string
	id                *string
	createdAt         *time.Time
	name              *string
	slug              *string
	description       *string
	logo_url          *string
	tos_urls          *[]string
	appendtos_urls    []string
	onion_urls        *[]string
	appendonion_urls  []string
	category          *string
	comments          *[]string
	appendcomments    []string
	tos_reviews       *[]schema.TosReview
	appendtos_reviews []schema.TosReview
	urls              *[]string
	appendurls        []string
	tags              *[]string
	appendtags        []string
	kyc_level         *int
	addkyc_level      *int
	_type             *service.Type
	listed            *bool
	verified          *bool
	pending           *bool
	xmr               *bool
	btc               *bool
	lightning         *bool
	cash              *bool
	fiat              *bool
	last_tos_review   *time.Time
	referral          *string
	score             *int
	addscore          *int
	potential_scam    *bool
	notes             *[]string
	appendnotes       []string
	support           *[]schema.SupportMethod
	appendsupport     []schema.SupportMethod
	clearedFields     map[string]struct{}
	attributes        map[string]struct{}
	removedattributes map[string]struct{}
	clearedattributes bool
	changes           map[int]struct{}
	removedchanges    map[int]struct{}
	clearedchanges    bool
	done              bool
	oldValue          func(context.Context) (*Service, error)
	predicates        []predicate.Service
}

var _ ent.Mutation = (*ServiceMutation)(nil)

// serviceOption allows management of the mutation configuration using functional options.
type serviceOption func(*ServiceMutation)

// newServiceMutation creates new mutation for the Service entity.
func newServiceMutation(c config, op Op, opts ...serviceOption) *ServiceMutation {
	m := &ServiceMutation{
		config:        c,
		op:            op,
		typ:           TypeService,
		clearedFields: make(map[string]struct{}),
	}
	for _, opt := range opts {
		opt(m)
	}
	return m
}

// withServiceID sets the ID field of the mutation.
func withServiceID(id string) serviceOption {
	return func(m *ServiceMutation) {
		var (
			err   error
			once  sync.Once
			value *Service
		)
		m.oldValue = func(ctx context.Context) (*Service, error) {
			once.Do(func() {
				if m.done {
					err = errors.New("querying old values post mutation is not allowed")
				} else {
					value, err = m.Client().Service.Get(ctx, id)
				}
			})
			return value, err
		}
		m.id = &id
	}
}

// withService sets the old Service of the mutation.
func withService(node *Service) serviceOption {
	return func(m *ServiceMutation) {
		m.oldValue = func(context.Context) (*Service, error) {
			return node, nil
		}
		m.id = &node.ID
	}
}

// Client returns a new `ent.Client` from the mutation. If the mutation was
// executed in a transaction (ent.Tx), a transactional client is returned.
func (m ServiceMutation) Client() *Client {
	client := &Client{config: m.config}
	client.init()
	return client
}

// Tx returns an `ent.Tx` for mutations that were executed in transactions;
// it returns an error otherwise.
func (m ServiceMutation) Tx() (*Tx, error) {
	if _, ok := m.driver.(*txDriver); !ok {
		return nil, errors.New("ent: mutation is not running in a transaction")
	}
	tx := &Tx{config: m.config}
	tx.init()
	return tx, nil
}

// SetID sets the value of the id field. Note that this
// operation is only accepted on creation of Service entities.
func (m *ServiceMutation) SetID(id string) {
	m.id = &id
}

// ID returns the ID value in the mutation. Note that the ID is only available
// if it was provided to the builder or after it was returned from the database.
func (m *ServiceMutation) ID() (id string, exists bool) {
	if m.id == nil {
		return
	}
	return *m.id, true
}

// IDs queries the database and returns the entity ids that match the mutation's predicate.
// That means, if the mutation is applied within a transaction with an isolation level such
// as sql.LevelSerializable, the returned ids match the ids of the rows that will be updated
// or updated by the mutation.
func (m *ServiceMutation) IDs(ctx context.Context) ([]string, error) {
	switch {
	case m.op.Is(OpUpdateOne | OpDeleteOne):
		id, exists := m.ID()
		if exists {
			return []string{id}, nil
		}
		fallthrough
	case m.op.Is(OpUpdate | OpDelete):
		return m.Client().Service.Query().Where(m.predicates...).IDs(ctx)
	default:
		return nil, fmt.Errorf("IDs is not allowed on %s operations", m.op)
	}
}

// SetCreatedAt sets the "createdAt" field.
func (m *ServiceMutation) SetCreatedAt(t time.Time) {
	m.createdAt = &t
}

// CreatedAt returns the value of the "createdAt" field in the mutation.
func (m *ServiceMutation) CreatedAt() (r time.Time, exists bool) {
	v := m.createdAt
	if v == nil {
		return
	}
	return *v, true
}

// OldCreatedAt returns the old "createdAt" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldCreatedAt(ctx context.Context) (v time.Time, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldCreatedAt is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldCreatedAt requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldCreatedAt: %w", err)
	}
	return oldValue.CreatedAt, nil
}

// ResetCreatedAt resets all changes to the "createdAt" field.
func (m *ServiceMutation) ResetCreatedAt() {
	m.createdAt = nil
}

// SetName sets the "name" field.
func (m *ServiceMutation) SetName(s string) {
	m.name = &s
}

// Name returns the value of the "name" field in the mutation.
func (m *ServiceMutation) Name() (r string, exists bool) {
	v := m.name
	if v == nil {
		return
	}
	return *v, true
}

// OldName returns the old "name" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldName(ctx context.Context) (v string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldName is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldName requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldName: %w", err)
	}
	return oldValue.Name, nil
}

// ResetName resets all changes to the "name" field.
func (m *ServiceMutation) ResetName() {
	m.name = nil
}

// SetSlug sets the "slug" field.
func (m *ServiceMutation) SetSlug(s string) {
	m.slug = &s
}

// Slug returns the value of the "slug" field in the mutation.
func (m *ServiceMutation) Slug() (r string, exists bool) {
	v := m.slug
	if v == nil {
		return
	}
	return *v, true
}

// OldSlug returns the old "slug" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldSlug(ctx context.Context) (v string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldSlug is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldSlug requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldSlug: %w", err)
	}
	return oldValue.Slug, nil
}

// ResetSlug resets all changes to the "slug" field.
func (m *ServiceMutation) ResetSlug() {
	m.slug = nil
}

// SetDescription sets the "description" field.
func (m *ServiceMutation) SetDescription(s string) {
	m.description = &s
}

// Description returns the value of the "description" field in the mutation.
func (m *ServiceMutation) Description() (r string, exists bool) {
	v := m.description
	if v == nil {
		return
	}
	return *v, true
}

// OldDescription returns the old "description" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldDescription(ctx context.Context) (v string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldDescription is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldDescription requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldDescription: %w", err)
	}
	return oldValue.Description, nil
}

// ResetDescription resets all changes to the "description" field.
func (m *ServiceMutation) ResetDescription() {
	m.description = nil
}

// SetLogoURL sets the "logo_url" field.
func (m *ServiceMutation) SetLogoURL(s string) {
	m.logo_url = &s
}

// LogoURL returns the value of the "logo_url" field in the mutation.
func (m *ServiceMutation) LogoURL() (r string, exists bool) {
	v := m.logo_url
	if v == nil {
		return
	}
	return *v, true
}

// OldLogoURL returns the old "logo_url" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldLogoURL(ctx context.Context) (v string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldLogoURL is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldLogoURL requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldLogoURL: %w", err)
	}
	return oldValue.LogoURL, nil
}

// ClearLogoURL clears the value of the "logo_url" field.
func (m *ServiceMutation) ClearLogoURL() {
	m.logo_url = nil
	m.clearedFields[service.FieldLogoURL] = struct{}{}
}

// LogoURLCleared returns if the "logo_url" field was cleared in this mutation.
func (m *ServiceMutation) LogoURLCleared() bool {
	_, ok := m.clearedFields[service.FieldLogoURL]
	return ok
}

// ResetLogoURL resets all changes to the "logo_url" field.
func (m *ServiceMutation) ResetLogoURL() {
	m.logo_url = nil
	delete(m.clearedFields, service.FieldLogoURL)
}

// SetTosUrls sets the "tos_urls" field.
func (m *ServiceMutation) SetTosUrls(s []string) {
	m.tos_urls = &s
	m.appendtos_urls = nil
}

// TosUrls returns the value of the "tos_urls" field in the mutation.
func (m *ServiceMutation) TosUrls() (r []string, exists bool) {
	v := m.tos_urls
	if v == nil {
		return
	}
	return *v, true
}

// OldTosUrls returns the old "tos_urls" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldTosUrls(ctx context.Context) (v []string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldTosUrls is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldTosUrls requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldTosUrls: %w", err)
	}
	return oldValue.TosUrls, nil
}

// AppendTosUrls adds s to the "tos_urls" field.
func (m *ServiceMutation) AppendTosUrls(s []string) {
	m.appendtos_urls = append(m.appendtos_urls, s...)
}

// AppendedTosUrls returns the list of values that were appended to the "tos_urls" field in this mutation.
func (m *ServiceMutation) AppendedTosUrls() ([]string, bool) {
	if len(m.appendtos_urls) == 0 {
		return nil, false
	}
	return m.appendtos_urls, true
}

// ClearTosUrls clears the value of the "tos_urls" field.
func (m *ServiceMutation) ClearTosUrls() {
	m.tos_urls = nil
	m.appendtos_urls = nil
	m.clearedFields[service.FieldTosUrls] = struct{}{}
}

// TosUrlsCleared returns if the "tos_urls" field was cleared in this mutation.
func (m *ServiceMutation) TosUrlsCleared() bool {
	_, ok := m.clearedFields[service.FieldTosUrls]
	return ok
}

// ResetTosUrls resets all changes to the "tos_urls" field.
func (m *ServiceMutation) ResetTosUrls() {
	m.tos_urls = nil
	m.appendtos_urls = nil
	delete(m.clearedFields, service.FieldTosUrls)
}

// SetOnionUrls sets the "onion_urls" field.
func (m *ServiceMutation) SetOnionUrls(s []string) {
	m.onion_urls = &s
	m.appendonion_urls = nil
}

// OnionUrls returns the value of the "onion_urls" field in the mutation.
func (m *ServiceMutation) OnionUrls() (r []string, exists bool) {
	v := m.onion_urls
	if v == nil {
		return
	}
	return *v, true
}

// OldOnionUrls returns the old "onion_urls" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldOnionUrls(ctx context.Context) (v []string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldOnionUrls is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldOnionUrls requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldOnionUrls: %w", err)
	}
	return oldValue.OnionUrls, nil
}

// AppendOnionUrls adds s to the "onion_urls" field.
func (m *ServiceMutation) AppendOnionUrls(s []string) {
	m.appendonion_urls = append(m.appendonion_urls, s...)
}

// AppendedOnionUrls returns the list of values that were appended to the "onion_urls" field in this mutation.
func (m *ServiceMutation) AppendedOnionUrls() ([]string, bool) {
	if len(m.appendonion_urls) == 0 {
		return nil, false
	}
	return m.appendonion_urls, true
}

// ClearOnionUrls clears the value of the "onion_urls" field.
func (m *ServiceMutation) ClearOnionUrls() {
	m.onion_urls = nil
	m.appendonion_urls = nil
	m.clearedFields[service.FieldOnionUrls] = struct{}{}
}

// OnionUrlsCleared returns if the "onion_urls" field was cleared in this mutation.
func (m *ServiceMutation) OnionUrlsCleared() bool {
	_, ok := m.clearedFields[service.FieldOnionUrls]
	return ok
}

// ResetOnionUrls resets all changes to the "onion_urls" field.
func (m *ServiceMutation) ResetOnionUrls() {
	m.onion_urls = nil
	m.appendonion_urls = nil
	delete(m.clearedFields, service.FieldOnionUrls)
}

// SetCategory sets the "category" field.
func (m *ServiceMutation) SetCategory(s string) {
	m.category = &s
}

// Category returns the value of the "category" field in the mutation.
func (m *ServiceMutation) Category() (r string, exists bool) {
	v := m.category
	if v == nil {
		return
	}
	return *v, true
}

// OldCategory returns the old "category" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldCategory(ctx context.Context) (v string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldCategory is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldCategory requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldCategory: %w", err)
	}
	return oldValue.Category, nil
}

// ClearCategory clears the value of the "category" field.
func (m *ServiceMutation) ClearCategory() {
	m.category = nil
	m.clearedFields[service.FieldCategory] = struct{}{}
}

// CategoryCleared returns if the "category" field was cleared in this mutation.
func (m *ServiceMutation) CategoryCleared() bool {
	_, ok := m.clearedFields[service.FieldCategory]
	return ok
}

// ResetCategory resets all changes to the "category" field.
func (m *ServiceMutation) ResetCategory() {
	m.category = nil
	delete(m.clearedFields, service.FieldCategory)
}

// SetComments sets the "comments" field.
func (m *ServiceMutation) SetComments(s []string) {
	m.comments = &s
	m.appendcomments = nil
}

// Comments returns the value of the "comments" field in the mutation.
func (m *ServiceMutation) Comments() (r []string, exists bool) {
	v := m.comments
	if v == nil {
		return
	}
	return *v, true
}

// OldComments returns the old "comments" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldComments(ctx context.Context) (v []string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldComments is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldComments requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldComments: %w", err)
	}
	return oldValue.Comments, nil
}

// AppendComments adds s to the "comments" field.
func (m *ServiceMutation) AppendComments(s []string) {
	m.appendcomments = append(m.appendcomments, s...)
}

// AppendedComments returns the list of values that were appended to the "comments" field in this mutation.
func (m *ServiceMutation) AppendedComments() ([]string, bool) {
	if len(m.appendcomments) == 0 {
		return nil, false
	}
	return m.appendcomments, true
}

// ClearComments clears the value of the "comments" field.
func (m *ServiceMutation) ClearComments() {
	m.comments = nil
	m.appendcomments = nil
	m.clearedFields[service.FieldComments] = struct{}{}
}

// CommentsCleared returns if the "comments" field was cleared in this mutation.
func (m *ServiceMutation) CommentsCleared() bool {
	_, ok := m.clearedFields[service.FieldComments]
	return ok
}

// ResetComments resets all changes to the "comments" field.
func (m *ServiceMutation) ResetComments() {
	m.comments = nil
	m.appendcomments = nil
	delete(m.clearedFields, service.FieldComments)
}

// SetTosReviews sets the "tos_reviews" field.
func (m *ServiceMutation) SetTosReviews(sr []schema.TosReview) {
	m.tos_reviews = &sr
	m.appendtos_reviews = nil
}

// TosReviews returns the value of the "tos_reviews" field in the mutation.
func (m *ServiceMutation) TosReviews() (r []schema.TosReview, exists bool) {
	v := m.tos_reviews
	if v == nil {
		return
	}
	return *v, true
}

// OldTosReviews returns the old "tos_reviews" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldTosReviews(ctx context.Context) (v []schema.TosReview, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldTosReviews is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldTosReviews requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldTosReviews: %w", err)
	}
	return oldValue.TosReviews, nil
}

// AppendTosReviews adds sr to the "tos_reviews" field.
func (m *ServiceMutation) AppendTosReviews(sr []schema.TosReview) {
	m.appendtos_reviews = append(m.appendtos_reviews, sr...)
}

// AppendedTosReviews returns the list of values that were appended to the "tos_reviews" field in this mutation.
func (m *ServiceMutation) AppendedTosReviews() ([]schema.TosReview, bool) {
	if len(m.appendtos_reviews) == 0 {
		return nil, false
	}
	return m.appendtos_reviews, true
}

// ClearTosReviews clears the value of the "tos_reviews" field.
func (m *ServiceMutation) ClearTosReviews() {
	m.tos_reviews = nil
	m.appendtos_reviews = nil
	m.clearedFields[service.FieldTosReviews] = struct{}{}
}

// TosReviewsCleared returns if the "tos_reviews" field was cleared in this mutation.
func (m *ServiceMutation) TosReviewsCleared() bool {
	_, ok := m.clearedFields[service.FieldTosReviews]
	return ok
}

// ResetTosReviews resets all changes to the "tos_reviews" field.
func (m *ServiceMutation) ResetTosReviews() {
	m.tos_reviews = nil
	m.appendtos_reviews = nil
	delete(m.clearedFields, service.FieldTosReviews)
}

// SetUrls sets the "urls" field.
func (m *ServiceMutation) SetUrls(s []string) {
	m.urls = &s
	m.appendurls = nil
}

// Urls returns the value of the "urls" field in the mutation.
func (m *ServiceMutation) Urls() (r []string, exists bool) {
	v := m.urls
	if v == nil {
		return
	}
	return *v, true
}

// OldUrls returns the old "urls" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldUrls(ctx context.Context) (v []string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldUrls is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldUrls requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldUrls: %w", err)
	}
	return oldValue.Urls, nil
}

// AppendUrls adds s to the "urls" field.
func (m *ServiceMutation) AppendUrls(s []string) {
	m.appendurls = append(m.appendurls, s...)
}

// AppendedUrls returns the list of values that were appended to the "urls" field in this mutation.
func (m *ServiceMutation) AppendedUrls() ([]string, bool) {
	if len(m.appendurls) == 0 {
		return nil, false
	}
	return m.appendurls, true
}

// ResetUrls resets all changes to the "urls" field.
func (m *ServiceMutation) ResetUrls() {
	m.urls = nil
	m.appendurls = nil
}

// SetTags sets the "tags" field.
func (m *ServiceMutation) SetTags(s []string) {
	m.tags = &s
	m.appendtags = nil
}

// Tags returns the value of the "tags" field in the mutation.
func (m *ServiceMutation) Tags() (r []string, exists bool) {
	v := m.tags
	if v == nil {
		return
	}
	return *v, true
}

// OldTags returns the old "tags" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldTags(ctx context.Context) (v []string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldTags is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldTags requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldTags: %w", err)
	}
	return oldValue.Tags, nil
}

// AppendTags adds s to the "tags" field.
func (m *ServiceMutation) AppendTags(s []string) {
	m.appendtags = append(m.appendtags, s...)
}

// AppendedTags returns the list of values that were appended to the "tags" field in this mutation.
func (m *ServiceMutation) AppendedTags() ([]string, bool) {
	if len(m.appendtags) == 0 {
		return nil, false
	}
	return m.appendtags, true
}

// ClearTags clears the value of the "tags" field.
func (m *ServiceMutation) ClearTags() {
	m.tags = nil
	m.appendtags = nil
	m.clearedFields[service.FieldTags] = struct{}{}
}

// TagsCleared returns if the "tags" field was cleared in this mutation.
func (m *ServiceMutation) TagsCleared() bool {
	_, ok := m.clearedFields[service.FieldTags]
	return ok
}

// ResetTags resets all changes to the "tags" field.
func (m *ServiceMutation) ResetTags() {
	m.tags = nil
	m.appendtags = nil
	delete(m.clearedFields, service.FieldTags)
}

// SetKycLevel sets the "kyc_level" field.
func (m *ServiceMutation) SetKycLevel(i int) {
	m.kyc_level = &i
	m.addkyc_level = nil
}

// KycLevel returns the value of the "kyc_level" field in the mutation.
func (m *ServiceMutation) KycLevel() (r int, exists bool) {
	v := m.kyc_level
	if v == nil {
		return
	}
	return *v, true
}

// OldKycLevel returns the old "kyc_level" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldKycLevel(ctx context.Context) (v int, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldKycLevel is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldKycLevel requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldKycLevel: %w", err)
	}
	return oldValue.KycLevel, nil
}

// AddKycLevel adds i to the "kyc_level" field.
func (m *ServiceMutation) AddKycLevel(i int) {
	if m.addkyc_level != nil {
		*m.addkyc_level += i
	} else {
		m.addkyc_level = &i
	}
}

// AddedKycLevel returns the value that was added to the "kyc_level" field in this mutation.
func (m *ServiceMutation) AddedKycLevel() (r int, exists bool) {
	v := m.addkyc_level
	if v == nil {
		return
	}
	return *v, true
}

// ClearKycLevel clears the value of the "kyc_level" field.
func (m *ServiceMutation) ClearKycLevel() {
	m.kyc_level = nil
	m.addkyc_level = nil
	m.clearedFields[service.FieldKycLevel] = struct{}{}
}

// KycLevelCleared returns if the "kyc_level" field was cleared in this mutation.
func (m *ServiceMutation) KycLevelCleared() bool {
	_, ok := m.clearedFields[service.FieldKycLevel]
	return ok
}

// ResetKycLevel resets all changes to the "kyc_level" field.
func (m *ServiceMutation) ResetKycLevel() {
	m.kyc_level = nil
	m.addkyc_level = nil
	delete(m.clearedFields, service.FieldKycLevel)
}

// SetType sets the "type" field.
func (m *ServiceMutation) SetType(s service.Type) {
	m._type = &s
}

// GetType returns the value of the "type" field in the mutation.
func (m *ServiceMutation) GetType() (r service.Type, exists bool) {
	v := m._type
	if v == nil {
		return
	}
	return *v, true
}

// OldType returns the old "type" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldType(ctx context.Context) (v service.Type, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldType is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldType requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldType: %w", err)
	}
	return oldValue.Type, nil
}

// ResetType resets all changes to the "type" field.
func (m *ServiceMutation) ResetType() {
	m._type = nil
}

// SetListed sets the "listed" field.
func (m *ServiceMutation) SetListed(b bool) {
	m.listed = &b
}

// Listed returns the value of the "listed" field in the mutation.
func (m *ServiceMutation) Listed() (r bool, exists bool) {
	v := m.listed
	if v == nil {
		return
	}
	return *v, true
}

// OldListed returns the old "listed" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldListed(ctx context.Context) (v bool, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldListed is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldListed requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldListed: %w", err)
	}
	return oldValue.Listed, nil
}

// ResetListed resets all changes to the "listed" field.
func (m *ServiceMutation) ResetListed() {
	m.listed = nil
}

// SetVerified sets the "verified" field.
func (m *ServiceMutation) SetVerified(b bool) {
	m.verified = &b
}

// Verified returns the value of the "verified" field in the mutation.
func (m *ServiceMutation) Verified() (r bool, exists bool) {
	v := m.verified
	if v == nil {
		return
	}
	return *v, true
}

// OldVerified returns the old "verified" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldVerified(ctx context.Context) (v bool, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldVerified is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldVerified requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldVerified: %w", err)
	}
	return oldValue.Verified, nil
}

// ResetVerified resets all changes to the "verified" field.
func (m *ServiceMutation) ResetVerified() {
	m.verified = nil
}

// SetPending sets the "pending" field.
func (m *ServiceMutation) SetPending(b bool) {
	m.pending = &b
}

// Pending returns the value of the "pending" field in the mutation.
func (m *ServiceMutation) Pending() (r bool, exists bool) {
	v := m.pending
	if v == nil {
		return
	}
	return *v, true
}

// OldPending returns the old "pending" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldPending(ctx context.Context) (v bool, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldPending is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldPending requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldPending: %w", err)
	}
	return oldValue.Pending, nil
}

// ResetPending resets all changes to the "pending" field.
func (m *ServiceMutation) ResetPending() {
	m.pending = nil
}

// SetXmr sets the "xmr" field.
func (m *ServiceMutation) SetXmr(b bool) {
	m.xmr = &b
}

// Xmr returns the value of the "xmr" field in the mutation.
func (m *ServiceMutation) Xmr() (r bool, exists bool) {
	v := m.xmr
	if v == nil {
		return
	}
	return *v, true
}

// OldXmr returns the old "xmr" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldXmr(ctx context.Context) (v bool, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldXmr is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldXmr requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldXmr: %w", err)
	}
	return oldValue.Xmr, nil
}

// ResetXmr resets all changes to the "xmr" field.
func (m *ServiceMutation) ResetXmr() {
	m.xmr = nil
}

// SetBtc sets the "btc" field.
func (m *ServiceMutation) SetBtc(b bool) {
	m.btc = &b
}

// Btc returns the value of the "btc" field in the mutation.
func (m *ServiceMutation) Btc() (r bool, exists bool) {
	v := m.btc
	if v == nil {
		return
	}
	return *v, true
}

// OldBtc returns the old "btc" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldBtc(ctx context.Context) (v bool, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldBtc is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldBtc requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldBtc: %w", err)
	}
	return oldValue.Btc, nil
}

// ResetBtc resets all changes to the "btc" field.
func (m *ServiceMutation) ResetBtc() {
	m.btc = nil
}

// SetLightning sets the "lightning" field.
func (m *ServiceMutation) SetLightning(b bool) {
	m.lightning = &b
}

// Lightning returns the value of the "lightning" field in the mutation.
func (m *ServiceMutation) Lightning() (r bool, exists bool) {
	v := m.lightning
	if v == nil {
		return
	}
	return *v, true
}

// OldLightning returns the old "lightning" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldLightning(ctx context.Context) (v bool, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldLightning is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldLightning requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldLightning: %w", err)
	}
	return oldValue.Lightning, nil
}

// ResetLightning resets all changes to the "lightning" field.
func (m *ServiceMutation) ResetLightning() {
	m.lightning = nil
}

// SetCash sets the "cash" field.
func (m *ServiceMutation) SetCash(b bool) {
	m.cash = &b
}

// Cash returns the value of the "cash" field in the mutation.
func (m *ServiceMutation) Cash() (r bool, exists bool) {
	v := m.cash
	if v == nil {
		return
	}
	return *v, true
}

// OldCash returns the old "cash" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldCash(ctx context.Context) (v bool, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldCash is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldCash requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldCash: %w", err)
	}
	return oldValue.Cash, nil
}

// ResetCash resets all changes to the "cash" field.
func (m *ServiceMutation) ResetCash() {
	m.cash = nil
}

// SetFiat sets the "fiat" field.
func (m *ServiceMutation) SetFiat(b bool) {
	m.fiat = &b
}

// Fiat returns the value of the "fiat" field in the mutation.
func (m *ServiceMutation) Fiat() (r bool, exists bool) {
	v := m.fiat
	if v == nil {
		return
	}
	return *v, true
}

// OldFiat returns the old "fiat" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldFiat(ctx context.Context) (v bool, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldFiat is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldFiat requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldFiat: %w", err)
	}
	return oldValue.Fiat, nil
}

// ResetFiat resets all changes to the "fiat" field.
func (m *ServiceMutation) ResetFiat() {
	m.fiat = nil
}

// SetLastTosReview sets the "last_tos_review" field.
func (m *ServiceMutation) SetLastTosReview(t time.Time) {
	m.last_tos_review = &t
}

// LastTosReview returns the value of the "last_tos_review" field in the mutation.
func (m *ServiceMutation) LastTosReview() (r time.Time, exists bool) {
	v := m.last_tos_review
	if v == nil {
		return
	}
	return *v, true
}

// OldLastTosReview returns the old "last_tos_review" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldLastTosReview(ctx context.Context) (v time.Time, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldLastTosReview is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldLastTosReview requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldLastTosReview: %w", err)
	}
	return oldValue.LastTosReview, nil
}

// ClearLastTosReview clears the value of the "last_tos_review" field.
func (m *ServiceMutation) ClearLastTosReview() {
	m.last_tos_review = nil
	m.clearedFields[service.FieldLastTosReview] = struct{}{}
}

// LastTosReviewCleared returns if the "last_tos_review" field was cleared in this mutation.
func (m *ServiceMutation) LastTosReviewCleared() bool {
	_, ok := m.clearedFields[service.FieldLastTosReview]
	return ok
}

// ResetLastTosReview resets all changes to the "last_tos_review" field.
func (m *ServiceMutation) ResetLastTosReview() {
	m.last_tos_review = nil
	delete(m.clearedFields, service.FieldLastTosReview)
}

// SetReferral sets the "referral" field.
func (m *ServiceMutation) SetReferral(s string) {
	m.referral = &s
}

// Referral returns the value of the "referral" field in the mutation.
func (m *ServiceMutation) Referral() (r string, exists bool) {
	v := m.referral
	if v == nil {
		return
	}
	return *v, true
}

// OldReferral returns the old "referral" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldReferral(ctx context.Context) (v string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldReferral is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldReferral requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldReferral: %w", err)
	}
	return oldValue.Referral, nil
}

// ClearReferral clears the value of the "referral" field.
func (m *ServiceMutation) ClearReferral() {
	m.referral = nil
	m.clearedFields[service.FieldReferral] = struct{}{}
}

// ReferralCleared returns if the "referral" field was cleared in this mutation.
func (m *ServiceMutation) ReferralCleared() bool {
	_, ok := m.clearedFields[service.FieldReferral]
	return ok
}

// ResetReferral resets all changes to the "referral" field.
func (m *ServiceMutation) ResetReferral() {
	m.referral = nil
	delete(m.clearedFields, service.FieldReferral)
}

// SetScore sets the "score" field.
func (m *ServiceMutation) SetScore(i int) {
	m.score = &i
	m.addscore = nil
}

// Score returns the value of the "score" field in the mutation.
func (m *ServiceMutation) Score() (r int, exists bool) {
	v := m.score
	if v == nil {
		return
	}
	return *v, true
}

// OldScore returns the old "score" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldScore(ctx context.Context) (v int, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldScore is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldScore requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldScore: %w", err)
	}
	return oldValue.Score, nil
}

// AddScore adds i to the "score" field.
func (m *ServiceMutation) AddScore(i int) {
	if m.addscore != nil {
		*m.addscore += i
	} else {
		m.addscore = &i
	}
}

// AddedScore returns the value that was added to the "score" field in this mutation.
func (m *ServiceMutation) AddedScore() (r int, exists bool) {
	v := m.addscore
	if v == nil {
		return
	}
	return *v, true
}

// ClearScore clears the value of the "score" field.
func (m *ServiceMutation) ClearScore() {
	m.score = nil
	m.addscore = nil
	m.clearedFields[service.FieldScore] = struct{}{}
}

// ScoreCleared returns if the "score" field was cleared in this mutation.
func (m *ServiceMutation) ScoreCleared() bool {
	_, ok := m.clearedFields[service.FieldScore]
	return ok
}

// ResetScore resets all changes to the "score" field.
func (m *ServiceMutation) ResetScore() {
	m.score = nil
	m.addscore = nil
	delete(m.clearedFields, service.FieldScore)
}

// SetPotentialScam sets the "potential_scam" field.
func (m *ServiceMutation) SetPotentialScam(b bool) {
	m.potential_scam = &b
}

// PotentialScam returns the value of the "potential_scam" field in the mutation.
func (m *ServiceMutation) PotentialScam() (r bool, exists bool) {
	v := m.potential_scam
	if v == nil {
		return
	}
	return *v, true
}

// OldPotentialScam returns the old "potential_scam" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldPotentialScam(ctx context.Context) (v bool, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldPotentialScam is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldPotentialScam requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldPotentialScam: %w", err)
	}
	return oldValue.PotentialScam, nil
}

// ResetPotentialScam resets all changes to the "potential_scam" field.
func (m *ServiceMutation) ResetPotentialScam() {
	m.potential_scam = nil
}

// SetNotes sets the "notes" field.
func (m *ServiceMutation) SetNotes(s []string) {
	m.notes = &s
	m.appendnotes = nil
}

// Notes returns the value of the "notes" field in the mutation.
func (m *ServiceMutation) Notes() (r []string, exists bool) {
	v := m.notes
	if v == nil {
		return
	}
	return *v, true
}

// OldNotes returns the old "notes" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldNotes(ctx context.Context) (v []string, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldNotes is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldNotes requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldNotes: %w", err)
	}
	return oldValue.Notes, nil
}

// AppendNotes adds s to the "notes" field.
func (m *ServiceMutation) AppendNotes(s []string) {
	m.appendnotes = append(m.appendnotes, s...)
}

// AppendedNotes returns the list of values that were appended to the "notes" field in this mutation.
func (m *ServiceMutation) AppendedNotes() ([]string, bool) {
	if len(m.appendnotes) == 0 {
		return nil, false
	}
	return m.appendnotes, true
}

// ClearNotes clears the value of the "notes" field.
func (m *ServiceMutation) ClearNotes() {
	m.notes = nil
	m.appendnotes = nil
	m.clearedFields[service.FieldNotes] = struct{}{}
}

// NotesCleared returns if the "notes" field was cleared in this mutation.
func (m *ServiceMutation) NotesCleared() bool {
	_, ok := m.clearedFields[service.FieldNotes]
	return ok
}

// ResetNotes resets all changes to the "notes" field.
func (m *ServiceMutation) ResetNotes() {
	m.notes = nil
	m.appendnotes = nil
	delete(m.clearedFields, service.FieldNotes)
}

// SetSupport sets the "support" field.
func (m *ServiceMutation) SetSupport(sm []schema.SupportMethod) {
	m.support = &sm
	m.appendsupport = nil
}

// Support returns the value of the "support" field in the mutation.
func (m *ServiceMutation) Support() (r []schema.SupportMethod, exists bool) {
	v := m.support
	if v == nil {
		return
	}
	return *v, true
}

// OldSupport returns the old "support" field's value of the Service entity.
// If the Service object wasn't provided to the builder, the object is fetched from the database.
// An error is returned if the mutation operation is not UpdateOne, or the database query fails.
func (m *ServiceMutation) OldSupport(ctx context.Context) (v []schema.SupportMethod, err error) {
	if !m.op.Is(OpUpdateOne) {
		return v, errors.New("OldSupport is only allowed on UpdateOne operations")
	}
	if m.id == nil || m.oldValue == nil {
		return v, errors.New("OldSupport requires an ID field in the mutation")
	}
	oldValue, err := m.oldValue(ctx)
	if err != nil {
		return v, fmt.Errorf("querying old value for OldSupport: %w", err)
	}
	return oldValue.Support, nil
}

// AppendSupport adds sm to the "support" field.
func (m *ServiceMutation) AppendSupport(sm []schema.SupportMethod) {
	m.appendsupport = append(m.appendsupport, sm...)
}

// AppendedSupport returns the list of values that were appended to the "support" field in this mutation.
func (m *ServiceMutation) AppendedSupport() ([]schema.SupportMethod, bool) {
	if len(m.appendsupport) == 0 {
		return nil, false
	}
	return m.appendsupport, true
}

// ClearSupport clears the value of the "support" field.
func (m *ServiceMutation) ClearSupport() {
	m.support = nil
	m.appendsupport = nil
	m.clearedFields[service.FieldSupport] = struct{}{}
}

// SupportCleared returns if the "support" field was cleared in this mutation.
func (m *ServiceMutation) SupportCleared() bool {
	_, ok := m.clearedFields[service.FieldSupport]
	return ok
}

// ResetSupport resets all changes to the "support" field.
func (m *ServiceMutation) ResetSupport() {
	m.support = nil
	m.appendsupport = nil
	delete(m.clearedFields, service.FieldSupport)
}

// AddAttributeIDs adds the "attributes" edge to the Attribute entity by ids.
func (m *ServiceMutation) AddAttributeIDs(ids ...string) {
	if m.attributes == nil {
		m.attributes = make(map[string]struct{})
	}
	for i := range ids {
		m.attributes[ids[i]] = struct{}{}
	}
}

// ClearAttributes clears the "attributes" edge to the Attribute entity.
func (m *ServiceMutation) ClearAttributes() {
	m.clearedattributes = true
}

// AttributesCleared reports if the "attributes" edge to the Attribute entity was cleared.
func (m *ServiceMutation) AttributesCleared() bool {
	return m.clearedattributes
}

// RemoveAttributeIDs removes the "attributes" edge to the Attribute entity by IDs.
func (m *ServiceMutation) RemoveAttributeIDs(ids ...string) {
	if m.removedattributes == nil {
		m.removedattributes = make(map[string]struct{})
	}
	for i := range ids {
		delete(m.attributes, ids[i])
		m.removedattributes[ids[i]] = struct{}{}
	}
}

// RemovedAttributes returns the removed IDs of the "attributes" edge to the Attribute entity.
func (m *ServiceMutation) RemovedAttributesIDs() (ids []string) {
	for id := range m.removedattributes {
		ids = append(ids, id)
	}
	return
}

// AttributesIDs returns the "attributes" edge IDs in the mutation.
func (m *ServiceMutation) AttributesIDs() (ids []string) {
	for id := range m.attributes {
		ids = append(ids, id)
	}
	return
}

// ResetAttributes resets all changes to the "attributes" edge.
func (m *ServiceMutation) ResetAttributes() {
	m.attributes = nil
	m.clearedattributes = false
	m.removedattributes = nil
}

// AddChangeIDs adds the "changes" edge to the Change entity by ids.
func (m *ServiceMutation) AddChangeIDs(ids ...int) {
	if m.changes == nil {
		m.changes = make(map[int]struct{})
	}
	for i := range ids {
		m.changes[ids[i]] = struct{}{}
	}
}

// ClearChanges clears the "changes" edge to the Change entity.
func (m *ServiceMutation) ClearChanges() {
	m.clearedchanges = true
}

// ChangesCleared reports if the "changes" edge to the Change entity was cleared.
func (m *ServiceMutation) ChangesCleared() bool {
	return m.clearedchanges
}

// RemoveChangeIDs removes the "changes" edge to the Change entity by IDs.
func (m *ServiceMutation) RemoveChangeIDs(ids ...int) {
	if m.removedchanges == nil {
		m.removedchanges = make(map[int]struct{})
	}
	for i := range ids {
		delete(m.changes, ids[i])
		m.removedchanges[ids[i]] = struct{}{}
	}
}

// RemovedChanges returns the removed IDs of the "changes" edge to the Change entity.
func (m *ServiceMutation) RemovedChangesIDs() (ids []int) {
	for id := range m.removedchanges {
		ids = append(ids, id)
	}
	return
}

// ChangesIDs returns the "changes" edge IDs in the mutation.
func (m *ServiceMutation) ChangesIDs() (ids []int) {
	for id := range m.changes {
		ids = append(ids, id)
	}
	return
}

// ResetChanges resets all changes to the "changes" edge.
func (m *ServiceMutation) ResetChanges() {
	m.changes = nil
	m.clearedchanges = false
	m.removedchanges = nil
}

// Where appends a list predicates to the ServiceMutation builder.
func (m *ServiceMutation) Where(ps ...predicate.Service) {
	m.predicates = append(m.predicates, ps...)
}

// WhereP appends storage-level predicates to the ServiceMutation builder. Using this method,
// users can use type-assertion to append predicates that do not depend on any generated package.
func (m *ServiceMutation) WhereP(ps ...func(*sql.Selector)) {
	p := make([]predicate.Service, len(ps))
	for i := range ps {
		p[i] = ps[i]
	}
	m.Where(p...)
}

// Op returns the operation name.
func (m *ServiceMutation) Op() Op {
	return m.op
}

// SetOp allows setting the mutation operation.
func (m *ServiceMutation) SetOp(op Op) {
	m.op = op
}

// Type returns the node type of this mutation (Service).
func (m *ServiceMutation) Type() string {
	return m.typ
}

// Fields returns all fields that were changed during this mutation. Note that in
// order to get all numeric fields that were incremented/decremented, call
// AddedFields().
func (m *ServiceMutation) Fields() []string {
	fields := make([]string, 0, 28)
	if m.createdAt != nil {
		fields = append(fields, service.FieldCreatedAt)
	}
	if m.name != nil {
		fields = append(fields, service.FieldName)
	}
	if m.slug != nil {
		fields = append(fields, service.FieldSlug)
	}
	if m.description != nil {
		fields = append(fields, service.FieldDescription)
	}
	if m.logo_url != nil {
		fields = append(fields, service.FieldLogoURL)
	}
	if m.tos_urls != nil {
		fields = append(fields, service.FieldTosUrls)
	}
	if m.onion_urls != nil {
		fields = append(fields, service.FieldOnionUrls)
	}
	if m.category != nil {
		fields = append(fields, service.FieldCategory)
	}
	if m.comments != nil {
		fields = append(fields, service.FieldComments)
	}
	if m.tos_reviews != nil {
		fields = append(fields, service.FieldTosReviews)
	}
	if m.urls != nil {
		fields = append(fields, service.FieldUrls)
	}
	if m.tags != nil {
		fields = append(fields, service.FieldTags)
	}
	if m.kyc_level != nil {
		fields = append(fields, service.FieldKycLevel)
	}
	if m._type != nil {
		fields = append(fields, service.FieldType)
	}
	if m.listed != nil {
		fields = append(fields, service.FieldListed)
	}
	if m.verified != nil {
		fields = append(fields, service.FieldVerified)
	}
	if m.pending != nil {
		fields = append(fields, service.FieldPending)
	}
	if m.xmr != nil {
		fields = append(fields, service.FieldXmr)
	}
	if m.btc != nil {
		fields = append(fields, service.FieldBtc)
	}
	if m.lightning != nil {
		fields = append(fields, service.FieldLightning)
	}
	if m.cash != nil {
		fields = append(fields, service.FieldCash)
	}
	if m.fiat != nil {
		fields = append(fields, service.FieldFiat)
	}
	if m.last_tos_review != nil {
		fields = append(fields, service.FieldLastTosReview)
	}
	if m.referral != nil {
		fields = append(fields, service.FieldReferral)
	}
	if m.score != nil {
		fields = append(fields, service.FieldScore)
	}
	if m.potential_scam != nil {
		fields = append(fields, service.FieldPotentialScam)
	}
	if m.notes != nil {
		fields = append(fields, service.FieldNotes)
	}
	if m.support != nil {
		fields = append(fields, service.FieldSupport)
	}
	return fields
}

// Field returns the value of a field with the given name. The second boolean
// return value indicates that this field was not set, or was not defined in the
// schema.
func (m *ServiceMutation) Field(name string) (ent.Value, bool) {
	switch name {
	case service.FieldCreatedAt:
		return m.CreatedAt()
	case service.FieldName:
		return m.Name()
	case service.FieldSlug:
		return m.Slug()
	case service.FieldDescription:
		return m.Description()
	case service.FieldLogoURL:
		return m.LogoURL()
	case service.FieldTosUrls:
		return m.TosUrls()
	case service.FieldOnionUrls:
		return m.OnionUrls()
	case service.FieldCategory:
		return m.Category()
	case service.FieldComments:
		return m.Comments()
	case service.FieldTosReviews:
		return m.TosReviews()
	case service.FieldUrls:
		return m.Urls()
	case service.FieldTags:
		return m.Tags()
	case service.FieldKycLevel:
		return m.KycLevel()
	case service.FieldType:
		return m.GetType()
	case service.FieldListed:
		return m.Listed()
	case service.FieldVerified:
		return m.Verified()
	case service.FieldPending:
		return m.Pending()
	case service.FieldXmr:
		return m.Xmr()
	case service.FieldBtc:
		return m.Btc()
	case service.FieldLightning:
		return m.Lightning()
	case service.FieldCash:
		return m.Cash()
	case service.FieldFiat:
		return m.Fiat()
	case service.FieldLastTosReview:
		return m.LastTosReview()
	case service.FieldReferral:
		return m.Referral()
	case service.FieldScore:
		return m.Score()
	case service.FieldPotentialScam:
		return m.PotentialScam()
	case service.FieldNotes:
		return m.Notes()
	case service.FieldSupport:
		return m.Support()
	}
	return nil, false
}

// OldField returns the old value of the field from the database. An error is
// returned if the mutation operation is not UpdateOne, or the query to the
// database failed.
func (m *ServiceMutation) OldField(ctx context.Context, name string) (ent.Value, error) {
	switch name {
	case service.FieldCreatedAt:
		return m.OldCreatedAt(ctx)
	case service.FieldName:
		return m.OldName(ctx)
	case service.FieldSlug:
		return m.OldSlug(ctx)
	case service.FieldDescription:
		return m.OldDescription(ctx)
	case service.FieldLogoURL:
		return m.OldLogoURL(ctx)
	case service.FieldTosUrls:
		return m.OldTosUrls(ctx)
	case service.FieldOnionUrls:
		return m.OldOnionUrls(ctx)
	case service.FieldCategory:
		return m.OldCategory(ctx)
	case service.FieldComments:
		return m.OldComments(ctx)
	case service.FieldTosReviews:
		return m.OldTosReviews(ctx)
	case service.FieldUrls:
		return m.OldUrls(ctx)
	case service.FieldTags:
		return m.OldTags(ctx)
	case service.FieldKycLevel:
		return m.OldKycLevel(ctx)
	case service.FieldType:
		return m.OldType(ctx)
	case service.FieldListed:
		return m.OldListed(ctx)
	case service.FieldVerified:
		return m.OldVerified(ctx)
	case service.FieldPending:
		return m.OldPending(ctx)
	case service.FieldXmr:
		return m.OldXmr(ctx)
	case service.FieldBtc:
		return m.OldBtc(ctx)
	case service.FieldLightning:
		return m.OldLightning(ctx)
	case service.FieldCash:
		return m.OldCash(ctx)
	case service.FieldFiat:
		return m.OldFiat(ctx)
	case service.FieldLastTosReview:
		return m.OldLastTosReview(ctx)
	case service.FieldReferral:
		return m.OldReferral(ctx)
	case service.FieldScore:
		return m.OldScore(ctx)
	case service.FieldPotentialScam:
		return m.OldPotentialScam(ctx)
	case service.FieldNotes:
		return m.OldNotes(ctx)
	case service.FieldSupport:
		return m.OldSupport(ctx)
	}
	return nil, fmt.Errorf("unknown Service field %s", name)
}

// SetField sets the value of a field with the given name. It returns an error if
// the field is not defined in the schema, or if the type mismatched the field
// type.
func (m *ServiceMutation) SetField(name string, value ent.Value) error {
	switch name {
	case service.FieldCreatedAt:
		v, ok := value.(time.Time)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetCreatedAt(v)
		return nil
	case service.FieldName:
		v, ok := value.(string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetName(v)
		return nil
	case service.FieldSlug:
		v, ok := value.(string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetSlug(v)
		return nil
	case service.FieldDescription:
		v, ok := value.(string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetDescription(v)
		return nil
	case service.FieldLogoURL:
		v, ok := value.(string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetLogoURL(v)
		return nil
	case service.FieldTosUrls:
		v, ok := value.([]string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetTosUrls(v)
		return nil
	case service.FieldOnionUrls:
		v, ok := value.([]string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetOnionUrls(v)
		return nil
	case service.FieldCategory:
		v, ok := value.(string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetCategory(v)
		return nil
	case service.FieldComments:
		v, ok := value.([]string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetComments(v)
		return nil
	case service.FieldTosReviews:
		v, ok := value.([]schema.TosReview)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetTosReviews(v)
		return nil
	case service.FieldUrls:
		v, ok := value.([]string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetUrls(v)
		return nil
	case service.FieldTags:
		v, ok := value.([]string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetTags(v)
		return nil
	case service.FieldKycLevel:
		v, ok := value.(int)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetKycLevel(v)
		return nil
	case service.FieldType:
		v, ok := value.(service.Type)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetType(v)
		return nil
	case service.FieldListed:
		v, ok := value.(bool)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetListed(v)
		return nil
	case service.FieldVerified:
		v, ok := value.(bool)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetVerified(v)
		return nil
	case service.FieldPending:
		v, ok := value.(bool)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetPending(v)
		return nil
	case service.FieldXmr:
		v, ok := value.(bool)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetXmr(v)
		return nil
	case service.FieldBtc:
		v, ok := value.(bool)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetBtc(v)
		return nil
	case service.FieldLightning:
		v, ok := value.(bool)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetLightning(v)
		return nil
	case service.FieldCash:
		v, ok := value.(bool)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetCash(v)
		return nil
	case service.FieldFiat:
		v, ok := value.(bool)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetFiat(v)
		return nil
	case service.FieldLastTosReview:
		v, ok := value.(time.Time)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetLastTosReview(v)
		return nil
	case service.FieldReferral:
		v, ok := value.(string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetReferral(v)
		return nil
	case service.FieldScore:
		v, ok := value.(int)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetScore(v)
		return nil
	case service.FieldPotentialScam:
		v, ok := value.(bool)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetPotentialScam(v)
		return nil
	case service.FieldNotes:
		v, ok := value.([]string)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetNotes(v)
		return nil
	case service.FieldSupport:
		v, ok := value.([]schema.SupportMethod)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.SetSupport(v)
		return nil
	}
	return fmt.Errorf("unknown Service field %s", name)
}

// AddedFields returns all numeric fields that were incremented/decremented during
// this mutation.
func (m *ServiceMutation) AddedFields() []string {
	var fields []string
	if m.addkyc_level != nil {
		fields = append(fields, service.FieldKycLevel)
	}
	if m.addscore != nil {
		fields = append(fields, service.FieldScore)
	}
	return fields
}

// AddedField returns the numeric value that was incremented/decremented on a field
// with the given name. The second boolean return value indicates that this field
// was not set, or was not defined in the schema.
func (m *ServiceMutation) AddedField(name string) (ent.Value, bool) {
	switch name {
	case service.FieldKycLevel:
		return m.AddedKycLevel()
	case service.FieldScore:
		return m.AddedScore()
	}
	return nil, false
}

// AddField adds the value to the field with the given name. It returns an error if
// the field is not defined in the schema, or if the type mismatched the field
// type.
func (m *ServiceMutation) AddField(name string, value ent.Value) error {
	switch name {
	case service.FieldKycLevel:
		v, ok := value.(int)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.AddKycLevel(v)
		return nil
	case service.FieldScore:
		v, ok := value.(int)
		if !ok {
			return fmt.Errorf("unexpected type %T for field %s", value, name)
		}
		m.AddScore(v)
		return nil
	}
	return fmt.Errorf("unknown Service numeric field %s", name)
}

// ClearedFields returns all nullable fields that were cleared during this
// mutation.
func (m *ServiceMutation) ClearedFields() []string {
	var fields []string
	if m.FieldCleared(service.FieldLogoURL) {
		fields = append(fields, service.FieldLogoURL)
	}
	if m.FieldCleared(service.FieldTosUrls) {
		fields = append(fields, service.FieldTosUrls)
	}
	if m.FieldCleared(service.FieldOnionUrls) {
		fields = append(fields, service.FieldOnionUrls)
	}
	if m.FieldCleared(service.FieldCategory) {
		fields = append(fields, service.FieldCategory)
	}
	if m.FieldCleared(service.FieldComments) {
		fields = append(fields, service.FieldComments)
	}
	if m.FieldCleared(service.FieldTosReviews) {
		fields = append(fields, service.FieldTosReviews)
	}
	if m.FieldCleared(service.FieldTags) {
		fields = append(fields, service.FieldTags)
	}
	if m.FieldCleared(service.FieldKycLevel) {
		fields = append(fields, service.FieldKycLevel)
	}
	if m.FieldCleared(service.FieldLastTosReview) {
		fields = append(fields, service.FieldLastTosReview)
	}
	if m.FieldCleared(service.FieldReferral) {
		fields = append(fields, service.FieldReferral)
	}
	if m.FieldCleared(service.FieldScore) {
		fields = append(fields, service.FieldScore)
	}
	if m.FieldCleared(service.FieldNotes) {
		fields = append(fields, service.FieldNotes)
	}
	if m.FieldCleared(service.FieldSupport) {
		fields = append(fields, service.FieldSupport)
	}
	return fields
}

// FieldCleared returns a boolean indicating if a field with the given name was
// cleared in this mutation.
func (m *ServiceMutation) FieldCleared(name string) bool {
	_, ok := m.clearedFields[name]
	return ok
}

// ClearField clears the value of the field with the given name. It returns an
// error if the field is not defined in the schema.
func (m *ServiceMutation) ClearField(name string) error {
	switch name {
	case service.FieldLogoURL:
		m.ClearLogoURL()
		return nil
	case service.FieldTosUrls:
		m.ClearTosUrls()
		return nil
	case service.FieldOnionUrls:
		m.ClearOnionUrls()
		return nil
	case service.FieldCategory:
		m.ClearCategory()
		return nil
	case service.FieldComments:
		m.ClearComments()
		return nil
	case service.FieldTosReviews:
		m.ClearTosReviews()
		return nil
	case service.FieldTags:
		m.ClearTags()
		return nil
	case service.FieldKycLevel:
		m.ClearKycLevel()
		return nil
	case service.FieldLastTosReview:
		m.ClearLastTosReview()
		return nil
	case service.FieldReferral:
		m.ClearReferral()
		return nil
	case service.FieldScore:
		m.ClearScore()
		return nil
	case service.FieldNotes:
		m.ClearNotes()
		return nil
	case service.FieldSupport:
		m.ClearSupport()
		return nil
	}
	return fmt.Errorf("unknown Service nullable field %s", name)
}

// ResetField resets all changes in the mutation for the field with the given name.
// It returns an error if the field is not defined in the schema.
func (m *ServiceMutation) ResetField(name string) error {
	switch name {
	case service.FieldCreatedAt:
		m.ResetCreatedAt()
		return nil
	case service.FieldName:
		m.ResetName()
		return nil
	case service.FieldSlug:
		m.ResetSlug()
		return nil
	case service.FieldDescription:
		m.ResetDescription()
		return nil
	case service.FieldLogoURL:
		m.ResetLogoURL()
		return nil
	case service.FieldTosUrls:
		m.ResetTosUrls()
		return nil
	case service.FieldOnionUrls:
		m.ResetOnionUrls()
		return nil
	case service.FieldCategory:
		m.ResetCategory()
		return nil
	case service.FieldComments:
		m.ResetComments()
		return nil
	case service.FieldTosReviews:
		m.ResetTosReviews()
		return nil
	case service.FieldUrls:
		m.ResetUrls()
		return nil
	case service.FieldTags:
		m.ResetTags()
		return nil
	case service.FieldKycLevel:
		m.ResetKycLevel()
		return nil
	case service.FieldType:
		m.ResetType()
		return nil
	case service.FieldListed:
		m.ResetListed()
		return nil
	case service.FieldVerified:
		m.ResetVerified()
		return nil
	case service.FieldPending:
		m.ResetPending()
		return nil
	case service.FieldXmr:
		m.ResetXmr()
		return nil
	case service.FieldBtc:
		m.ResetBtc()
		return nil
	case service.FieldLightning:
		m.ResetLightning()
		return nil
	case service.FieldCash:
		m.ResetCash()
		return nil
	case service.FieldFiat:
		m.ResetFiat()
		return nil
	case service.FieldLastTosReview:
		m.ResetLastTosReview()
		return nil
	case service.FieldReferral:
		m.ResetReferral()
		return nil
	case service.FieldScore:
		m.ResetScore()
		return nil
	case service.FieldPotentialScam:
		m.ResetPotentialScam()
		return nil
	case service.FieldNotes:
		m.ResetNotes()
		return nil
	case service.FieldSupport:
		m.ResetSupport()
		return nil
	}
	return fmt.Errorf("unknown Service field %s", name)
}

// AddedEdges returns all edge names that were set/added in this mutation.
func (m *ServiceMutation) AddedEdges() []string {
	edges := make([]string, 0, 2)
	if m.attributes != nil {
		edges = append(edges, service.EdgeAttributes)
	}
	if m.changes != nil {
		edges = append(edges, service.EdgeChanges)
	}
	return edges
}

// AddedIDs returns all IDs (to other nodes) that were added for the given edge
// name in this mutation.
func (m *ServiceMutation) AddedIDs(name string) []ent.Value {
	switch name {
	case service.EdgeAttributes:
		ids := make([]ent.Value, 0, len(m.attributes))
		for id := range m.attributes {
			ids = append(ids, id)
		}
		return ids
	case service.EdgeChanges:
		ids := make([]ent.Value, 0, len(m.changes))
		for id := range m.changes {
			ids = append(ids, id)
		}
		return ids
	}
	return nil
}

// RemovedEdges returns all edge names that were removed in this mutation.
func (m *ServiceMutation) RemovedEdges() []string {
	edges := make([]string, 0, 2)
	if m.removedattributes != nil {
		edges = append(edges, service.EdgeAttributes)
	}
	if m.removedchanges != nil {
		edges = append(edges, service.EdgeChanges)
	}
	return edges
}

// RemovedIDs returns all IDs (to other nodes) that were removed for the edge with
// the given name in this mutation.
func (m *ServiceMutation) RemovedIDs(name string) []ent.Value {
	switch name {
	case service.EdgeAttributes:
		ids := make([]ent.Value, 0, len(m.removedattributes))
		for id := range m.removedattributes {
			ids = append(ids, id)
		}
		return ids
	case service.EdgeChanges:
		ids := make([]ent.Value, 0, len(m.removedchanges))
		for id := range m.removedchanges {
			ids = append(ids, id)
		}
		return ids
	}
	return nil
}

// ClearedEdges returns all edge names that were cleared in this mutation.
func (m *ServiceMutation) ClearedEdges() []string {
	edges := make([]string, 0, 2)
	if m.clearedattributes {
		edges = append(edges, service.EdgeAttributes)
	}
	if m.clearedchanges {
		edges = append(edges, service.EdgeChanges)
	}
	return edges
}

// EdgeCleared returns a boolean which indicates if the edge with the given name
// was cleared in this mutation.
func (m *ServiceMutation) EdgeCleared(name string) bool {
	switch name {
	case service.EdgeAttributes:
		return m.clearedattributes
	case service.EdgeChanges:
		return m.clearedchanges
	}
	return false
}

// ClearEdge clears the value of the edge with the given name. It returns an error
// if that edge is not defined in the schema.
func (m *ServiceMutation) ClearEdge(name string) error {
	switch name {
	}
	return fmt.Errorf("unknown Service unique edge %s", name)
}

// ResetEdge resets all changes to the edge with the given name in this mutation.
// It returns an error if the edge is not defined in the schema.
func (m *ServiceMutation) ResetEdge(name string) error {
	switch name {
	case service.EdgeAttributes:
		m.ResetAttributes()
		return nil
	case service.EdgeChanges:
		m.ResetChanges()
		return nil
	}
	return fmt.Errorf("unknown Service edge %s", name)
}
