// Code generated by ent, DO NOT EDIT.

package ent

import (
	"encoding/json"
	"fmt"
	"strings"
	"time"

	"entgo.io/ent"
	"entgo.io/ent/dialect/sql"
	"kycnot.me/internal/ent/schema"
	"kycnot.me/internal/ent/service"
)

// Service is the model entity for the Service schema.
type Service struct {
	config `json:"-"`
	// ID of the ent.
	ID string `json:"id"`
	// CreatedAt holds the value of the "createdAt" field.
	CreatedAt time.Time `json:"createdAt"`
	// The name of the service
	Name string `json:"name,omitempty"`
	// The slug of the service
	Slug string `json:"slug,omitempty"`
	// Description of the service
	Description string `json:"description"`
	// URL of the service logo
	LogoURL string `json:"logo_url,omitempty"`
	// Terms of Service URLs
	TosUrls []string `json:"tos_urls,omitempty"`
	// Onion (Tor) URLs for the service
	OnionUrls []string `json:"onion_urls,omitempty"`
	// Category of the service
	Category string `json:"category,omitempty"`
	// User comments about the service
	Comments []string `json:"comments,omitempty"`
	// Reviews of the Terms of Service
	TosReviews []schema.TosReview `json:"tos_reviews,omitempty"`
	// URLs associated with the service
	Urls []string `json:"urls"`
	// Tags associated with the service
	Tags []string `json:"tags,omitempty"`
	// Know Your Customer level (0-3)
	KycLevel int `json:"kyc_level,omitempty"`
	// Type of the service
	Type service.Type `json:"type,omitempty"`
	// Whether the service is listed
	Listed bool `json:"listed,omitempty"`
	// Whether the service is verified
	Verified bool `json:"verified,omitempty"`
	// Whether the service is pending approval
	Pending bool `json:"pending,omitempty"`
	// Supports Monero (XMR)
	Xmr bool `json:"xmr,omitempty"`
	// Supports Bitcoin (BTC)
	Btc bool `json:"btc,omitempty"`
	// Supports Lightning Network
	Lightning bool `json:"lightning,omitempty"`
	// Supports cash transactions
	Cash bool `json:"cash,omitempty"`
	// Supports fiat currency
	Fiat bool `json:"fiat,omitempty"`
	// Date of the last Terms of Service review
	LastTosReview time.Time `json:"last_tos_review,omitempty"`
	// Referral information
	Referral string `json:"referral,omitempty"`
	// Service score (0-10)
	Score int `json:"score,omitempty"`
	// Flag for potential scam
	PotentialScam bool `json:"potential_scam,omitempty"`
	// Additional notes about the service
	Notes []string `json:"notes,omitempty"`
	// Support information
	Support []schema.SupportMethod `json:"support,omitempty"`
	// Edges holds the relations/edges for other nodes in the graph.
	// The values are being populated by the ServiceQuery when eager-loading is set.
	Edges        ServiceEdges `json:"edges"`
	selectValues sql.SelectValues
}

// ServiceEdges holds the relations/edges for other nodes in the graph.
type ServiceEdges struct {
	// Attributes associated with the service
	Attributes []*Attribute `json:"attributes,omitempty"`
	// Changes associated with the service
	Changes []*Change `json:"changes,omitempty"`
	// loadedTypes holds the information for reporting if a
	// type was loaded (or requested) in eager-loading or not.
	loadedTypes [2]bool
}

// AttributesOrErr returns the Attributes value or an error if the edge
// was not loaded in eager-loading.
func (e ServiceEdges) AttributesOrErr() ([]*Attribute, error) {
	if e.loadedTypes[0] {
		return e.Attributes, nil
	}
	return nil, &NotLoadedError{edge: "attributes"}
}

// ChangesOrErr returns the Changes value or an error if the edge
// was not loaded in eager-loading.
func (e ServiceEdges) ChangesOrErr() ([]*Change, error) {
	if e.loadedTypes[1] {
		return e.Changes, nil
	}
	return nil, &NotLoadedError{edge: "changes"}
}

// scanValues returns the types for scanning values from sql.Rows.
func (*Service) scanValues(columns []string) ([]any, error) {
	values := make([]any, len(columns))
	for i := range columns {
		switch columns[i] {
		case service.FieldTosUrls, service.FieldOnionUrls, service.FieldComments, service.FieldTosReviews, service.FieldUrls, service.FieldTags, service.FieldNotes, service.FieldSupport:
			values[i] = new([]byte)
		case service.FieldListed, service.FieldVerified, service.FieldPending, service.FieldXmr, service.FieldBtc, service.FieldLightning, service.FieldCash, service.FieldFiat, service.FieldPotentialScam:
			values[i] = new(sql.NullBool)
		case service.FieldKycLevel, service.FieldScore:
			values[i] = new(sql.NullInt64)
		case service.FieldID, service.FieldName, service.FieldSlug, service.FieldDescription, service.FieldLogoURL, service.FieldCategory, service.FieldType, service.FieldReferral:
			values[i] = new(sql.NullString)
		case service.FieldCreatedAt, service.FieldLastTosReview:
			values[i] = new(sql.NullTime)
		default:
			values[i] = new(sql.UnknownType)
		}
	}
	return values, nil
}

// assignValues assigns the values that were returned from sql.Rows (after scanning)
// to the Service fields.
func (s *Service) assignValues(columns []string, values []any) error {
	if m, n := len(values), len(columns); m < n {
		return fmt.Errorf("mismatch number of scan values: %d != %d", m, n)
	}
	for i := range columns {
		switch columns[i] {
		case service.FieldID:
			if value, ok := values[i].(*sql.NullString); !ok {
				return fmt.Errorf("unexpected type %T for field id", values[i])
			} else if value.Valid {
				s.ID = value.String
			}
		case service.FieldCreatedAt:
			if value, ok := values[i].(*sql.NullTime); !ok {
				return fmt.Errorf("unexpected type %T for field createdAt", values[i])
			} else if value.Valid {
				s.CreatedAt = value.Time
			}
		case service.FieldName:
			if value, ok := values[i].(*sql.NullString); !ok {
				return fmt.Errorf("unexpected type %T for field name", values[i])
			} else if value.Valid {
				s.Name = value.String
			}
		case service.FieldSlug:
			if value, ok := values[i].(*sql.NullString); !ok {
				return fmt.Errorf("unexpected type %T for field slug", values[i])
			} else if value.Valid {
				s.Slug = value.String
			}
		case service.FieldDescription:
			if value, ok := values[i].(*sql.NullString); !ok {
				return fmt.Errorf("unexpected type %T for field description", values[i])
			} else if value.Valid {
				s.Description = value.String
			}
		case service.FieldLogoURL:
			if value, ok := values[i].(*sql.NullString); !ok {
				return fmt.Errorf("unexpected type %T for field logo_url", values[i])
			} else if value.Valid {
				s.LogoURL = value.String
			}
		case service.FieldTosUrls:
			if value, ok := values[i].(*[]byte); !ok {
				return fmt.Errorf("unexpected type %T for field tos_urls", values[i])
			} else if value != nil && len(*value) > 0 {
				if err := json.Unmarshal(*value, &s.TosUrls); err != nil {
					return fmt.Errorf("unmarshal field tos_urls: %w", err)
				}
			}
		case service.FieldOnionUrls:
			if value, ok := values[i].(*[]byte); !ok {
				return fmt.Errorf("unexpected type %T for field onion_urls", values[i])
			} else if value != nil && len(*value) > 0 {
				if err := json.Unmarshal(*value, &s.OnionUrls); err != nil {
					return fmt.Errorf("unmarshal field onion_urls: %w", err)
				}
			}
		case service.FieldCategory:
			if value, ok := values[i].(*sql.NullString); !ok {
				return fmt.Errorf("unexpected type %T for field category", values[i])
			} else if value.Valid {
				s.Category = value.String
			}
		case service.FieldComments:
			if value, ok := values[i].(*[]byte); !ok {
				return fmt.Errorf("unexpected type %T for field comments", values[i])
			} else if value != nil && len(*value) > 0 {
				if err := json.Unmarshal(*value, &s.Comments); err != nil {
					return fmt.Errorf("unmarshal field comments: %w", err)
				}
			}
		case service.FieldTosReviews:
			if value, ok := values[i].(*[]byte); !ok {
				return fmt.Errorf("unexpected type %T for field tos_reviews", values[i])
			} else if value != nil && len(*value) > 0 {
				if err := json.Unmarshal(*value, &s.TosReviews); err != nil {
					return fmt.Errorf("unmarshal field tos_reviews: %w", err)
				}
			}
		case service.FieldUrls:
			if value, ok := values[i].(*[]byte); !ok {
				return fmt.Errorf("unexpected type %T for field urls", values[i])
			} else if value != nil && len(*value) > 0 {
				if err := json.Unmarshal(*value, &s.Urls); err != nil {
					return fmt.Errorf("unmarshal field urls: %w", err)
				}
			}
		case service.FieldTags:
			if value, ok := values[i].(*[]byte); !ok {
				return fmt.Errorf("unexpected type %T for field tags", values[i])
			} else if value != nil && len(*value) > 0 {
				if err := json.Unmarshal(*value, &s.Tags); err != nil {
					return fmt.Errorf("unmarshal field tags: %w", err)
				}
			}
		case service.FieldKycLevel:
			if value, ok := values[i].(*sql.NullInt64); !ok {
				return fmt.Errorf("unexpected type %T for field kyc_level", values[i])
			} else if value.Valid {
				s.KycLevel = int(value.Int64)
			}
		case service.FieldType:
			if value, ok := values[i].(*sql.NullString); !ok {
				return fmt.Errorf("unexpected type %T for field type", values[i])
			} else if value.Valid {
				s.Type = service.Type(value.String)
			}
		case service.FieldListed:
			if value, ok := values[i].(*sql.NullBool); !ok {
				return fmt.Errorf("unexpected type %T for field listed", values[i])
			} else if value.Valid {
				s.Listed = value.Bool
			}
		case service.FieldVerified:
			if value, ok := values[i].(*sql.NullBool); !ok {
				return fmt.Errorf("unexpected type %T for field verified", values[i])
			} else if value.Valid {
				s.Verified = value.Bool
			}
		case service.FieldPending:
			if value, ok := values[i].(*sql.NullBool); !ok {
				return fmt.Errorf("unexpected type %T for field pending", values[i])
			} else if value.Valid {
				s.Pending = value.Bool
			}
		case service.FieldXmr:
			if value, ok := values[i].(*sql.NullBool); !ok {
				return fmt.Errorf("unexpected type %T for field xmr", values[i])
			} else if value.Valid {
				s.Xmr = value.Bool
			}
		case service.FieldBtc:
			if value, ok := values[i].(*sql.NullBool); !ok {
				return fmt.Errorf("unexpected type %T for field btc", values[i])
			} else if value.Valid {
				s.Btc = value.Bool
			}
		case service.FieldLightning:
			if value, ok := values[i].(*sql.NullBool); !ok {
				return fmt.Errorf("unexpected type %T for field lightning", values[i])
			} else if value.Valid {
				s.Lightning = value.Bool
			}
		case service.FieldCash:
			if value, ok := values[i].(*sql.NullBool); !ok {
				return fmt.Errorf("unexpected type %T for field cash", values[i])
			} else if value.Valid {
				s.Cash = value.Bool
			}
		case service.FieldFiat:
			if value, ok := values[i].(*sql.NullBool); !ok {
				return fmt.Errorf("unexpected type %T for field fiat", values[i])
			} else if value.Valid {
				s.Fiat = value.Bool
			}
		case service.FieldLastTosReview:
			if value, ok := values[i].(*sql.NullTime); !ok {
				return fmt.Errorf("unexpected type %T for field last_tos_review", values[i])
			} else if value.Valid {
				s.LastTosReview = value.Time
			}
		case service.FieldReferral:
			if value, ok := values[i].(*sql.NullString); !ok {
				return fmt.Errorf("unexpected type %T for field referral", values[i])
			} else if value.Valid {
				s.Referral = value.String
			}
		case service.FieldScore:
			if value, ok := values[i].(*sql.NullInt64); !ok {
				return fmt.Errorf("unexpected type %T for field score", values[i])
			} else if value.Valid {
				s.Score = int(value.Int64)
			}
		case service.FieldPotentialScam:
			if value, ok := values[i].(*sql.NullBool); !ok {
				return fmt.Errorf("unexpected type %T for field potential_scam", values[i])
			} else if value.Valid {
				s.PotentialScam = value.Bool
			}
		case service.FieldNotes:
			if value, ok := values[i].(*[]byte); !ok {
				return fmt.Errorf("unexpected type %T for field notes", values[i])
			} else if value != nil && len(*value) > 0 {
				if err := json.Unmarshal(*value, &s.Notes); err != nil {
					return fmt.Errorf("unmarshal field notes: %w", err)
				}
			}
		case service.FieldSupport:
			if value, ok := values[i].(*[]byte); !ok {
				return fmt.Errorf("unexpected type %T for field support", values[i])
			} else if value != nil && len(*value) > 0 {
				if err := json.Unmarshal(*value, &s.Support); err != nil {
					return fmt.Errorf("unmarshal field support: %w", err)
				}
			}
		default:
			s.selectValues.Set(columns[i], values[i])
		}
	}
	return nil
}

// Value returns the ent.Value that was dynamically selected and assigned to the Service.
// This includes values selected through modifiers, order, etc.
func (s *Service) Value(name string) (ent.Value, error) {
	return s.selectValues.Get(name)
}

// QueryAttributes queries the "attributes" edge of the Service entity.
func (s *Service) QueryAttributes() *AttributeQuery {
	return NewServiceClient(s.config).QueryAttributes(s)
}

// QueryChanges queries the "changes" edge of the Service entity.
func (s *Service) QueryChanges() *ChangeQuery {
	return NewServiceClient(s.config).QueryChanges(s)
}

// Update returns a builder for updating this Service.
// Note that you need to call Service.Unwrap() before calling this method if this Service
// was returned from a transaction, and the transaction was committed or rolled back.
func (s *Service) Update() *ServiceUpdateOne {
	return NewServiceClient(s.config).UpdateOne(s)
}

// Unwrap unwraps the Service entity that was returned from a transaction after it was closed,
// so that all future queries will be executed through the driver which created the transaction.
func (s *Service) Unwrap() *Service {
	_tx, ok := s.config.driver.(*txDriver)
	if !ok {
		panic("ent: Service is not a transactional entity")
	}
	s.config.driver = _tx.drv
	return s
}

// String implements the fmt.Stringer.
func (s *Service) String() string {
	var builder strings.Builder
	builder.WriteString("Service(")
	builder.WriteString(fmt.Sprintf("id=%v, ", s.ID))
	builder.WriteString("createdAt=")
	builder.WriteString(s.CreatedAt.Format(time.ANSIC))
	builder.WriteString(", ")
	builder.WriteString("name=")
	builder.WriteString(s.Name)
	builder.WriteString(", ")
	builder.WriteString("slug=")
	builder.WriteString(s.Slug)
	builder.WriteString(", ")
	builder.WriteString("description=")
	builder.WriteString(s.Description)
	builder.WriteString(", ")
	builder.WriteString("logo_url=")
	builder.WriteString(s.LogoURL)
	builder.WriteString(", ")
	builder.WriteString("tos_urls=")
	builder.WriteString(fmt.Sprintf("%v", s.TosUrls))
	builder.WriteString(", ")
	builder.WriteString("onion_urls=")
	builder.WriteString(fmt.Sprintf("%v", s.OnionUrls))
	builder.WriteString(", ")
	builder.WriteString("category=")
	builder.WriteString(s.Category)
	builder.WriteString(", ")
	builder.WriteString("comments=")
	builder.WriteString(fmt.Sprintf("%v", s.Comments))
	builder.WriteString(", ")
	builder.WriteString("tos_reviews=")
	builder.WriteString(fmt.Sprintf("%v", s.TosReviews))
	builder.WriteString(", ")
	builder.WriteString("urls=")
	builder.WriteString(fmt.Sprintf("%v", s.Urls))
	builder.WriteString(", ")
	builder.WriteString("tags=")
	builder.WriteString(fmt.Sprintf("%v", s.Tags))
	builder.WriteString(", ")
	builder.WriteString("kyc_level=")
	builder.WriteString(fmt.Sprintf("%v", s.KycLevel))
	builder.WriteString(", ")
	builder.WriteString("type=")
	builder.WriteString(fmt.Sprintf("%v", s.Type))
	builder.WriteString(", ")
	builder.WriteString("listed=")
	builder.WriteString(fmt.Sprintf("%v", s.Listed))
	builder.WriteString(", ")
	builder.WriteString("verified=")
	builder.WriteString(fmt.Sprintf("%v", s.Verified))
	builder.WriteString(", ")
	builder.WriteString("pending=")
	builder.WriteString(fmt.Sprintf("%v", s.Pending))
	builder.WriteString(", ")
	builder.WriteString("xmr=")
	builder.WriteString(fmt.Sprintf("%v", s.Xmr))
	builder.WriteString(", ")
	builder.WriteString("btc=")
	builder.WriteString(fmt.Sprintf("%v", s.Btc))
	builder.WriteString(", ")
	builder.WriteString("lightning=")
	builder.WriteString(fmt.Sprintf("%v", s.Lightning))
	builder.WriteString(", ")
	builder.WriteString("cash=")
	builder.WriteString(fmt.Sprintf("%v", s.Cash))
	builder.WriteString(", ")
	builder.WriteString("fiat=")
	builder.WriteString(fmt.Sprintf("%v", s.Fiat))
	builder.WriteString(", ")
	builder.WriteString("last_tos_review=")
	builder.WriteString(s.LastTosReview.Format(time.ANSIC))
	builder.WriteString(", ")
	builder.WriteString("referral=")
	builder.WriteString(s.Referral)
	builder.WriteString(", ")
	builder.WriteString("score=")
	builder.WriteString(fmt.Sprintf("%v", s.Score))
	builder.WriteString(", ")
	builder.WriteString("potential_scam=")
	builder.WriteString(fmt.Sprintf("%v", s.PotentialScam))
	builder.WriteString(", ")
	builder.WriteString("notes=")
	builder.WriteString(fmt.Sprintf("%v", s.Notes))
	builder.WriteString(", ")
	builder.WriteString("support=")
	builder.WriteString(fmt.Sprintf("%v", s.Support))
	builder.WriteByte(')')
	return builder.String()
}

// Services is a parsable slice of Service.
type Services []*Service
